// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.0.

#ifndef __P5TET_H
#define __P5TET_H

#include <cmath>
#include <stdexcept>
#include <ufc.h>

/// This class defines the interface for a finite element.

class P5tet_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  P5tet_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~P5tet_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.666666667*x*psitilde_a_2 - 0.6666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.02857142857*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.7714285714*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.01851851852*psitilde_bs_0_4 + 1.833333333*y*psitilde_bs_0_4 - 0.8148148148*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.2857142857*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.7142857143*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.1785714286*psitilde_bs_1_3 + 1.964285714*y*psitilde_bs_1_3 - 0.7857142857*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.020408163*psitilde_bs_2_1 + 2.571428571*y*psitilde_bs_2_1 - 0.5510204082*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.5787037037*psitilde_bs_2_2 + 2.291666667*y*psitilde_bs_2_2 - 0.712962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.512345679*psitilde_bs_3_1 + 3.055555556*y*psitilde_bs_3_1 - 0.5432098765*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.1555555556*psitilde_cs_00_2 + 1.866666667*z*psitilde_cs_00_2 - 0.7111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.06285714286*psitilde_cs_00_4 + 1.885714286*z*psitilde_cs_00_4 - 0.8228571429*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.7777777778*psitilde_cs_01_1 + 2.333333333*z*psitilde_cs_01_1 - 0.5555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.4285714286*psitilde_cs_01_2 + 2.142857143*z*psitilde_cs_01_2 - 0.7142857143*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.7333333333*psitilde_cs_02_2 + 2.444444444*z*psitilde_cs_02_2 - 0.7111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.7777777778*psitilde_cs_10_1 + 2.333333333*z*psitilde_cs_10_1 - 0.5555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.4285714286*psitilde_cs_10_2 + 2.142857143*z*psitilde_cs_10_2 - 0.7142857143*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.7333333333*psitilde_cs_11_2 + 2.444444444*z*psitilde_cs_11_2 - 0.7111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.7333333333*psitilde_cs_20_2 + 2.444444444*z*psitilde_cs_20_2 - 0.7111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.937253933*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.708203932*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.196152423*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.809475019*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.598076211*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.674234614*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.121320344*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.12429773*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.810708435*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.291561976*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.422616289*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.708099244*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.774964387*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.416198487*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.744562647*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.31662479*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.422616289*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.974937186*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.872281323*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.062019202*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520788*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.658312395*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.64581852*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476413*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.68332145*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.874208829*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.415880433*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.09338662*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.013878189*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.982120022*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.031128874*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.539392014*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.062257748*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.244997998*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.605551275*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.982120022*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.408326913*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.122498999*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.415880433*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.549509757*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.802775638*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.009450674049, 0.006097881207, 0.003520613356, 0.002489449578, 0.007059462333, 0.00546823601, 0.003157087532, 0.004464796006, 0.002577751176, 0.001822745337, -0.002982674187, -0.00252081978, -0.001952618605, -0.001127344877, -0.002183093968, -0.001691017316, -0.0009763093026, -0.001380709857, -0.0007971532075, -0.0005636724387, 0.002786871125, 0.002457789311, 0.002077211093, 0.001609000794, 0.0009289570415, 0.002198313589, 0.001857914083, 0.00143913406, 0.0008308844372, 0.001609000794, 0.001246326656, 0.0007195670302, 0.001017621453, 0.0005875240199, 0.0004154422186, -0.007055935247, -0.006382333571, -0.005628689138, -0.004757110573, -0.003684842005, -0.002127444524, -0.005826246777, -0.005138266683, -0.004342627949, -0.003363785145, -0.001942082259, -0.004595805436, -0.003884164518, -0.003008660899, -0.00173705118, -0.003363785145, -0.002605576769, -0.001504330449, -0.002127444524, -0.001228280668, -0.0008685255898},
    {0.009450674049, -0.006097881207, 0.003520613356, 0.002489449578, 0.007059462333, -0.00546823601, 0.003157087532, -0.004464796006, 0.002577751176, 0.001822745337, 0.002982674187, -0.00252081978, 0.001952618605, -0.001127344877, -0.002183093968, 0.001691017316, -0.0009763093026, 0.001380709857, -0.0007971532075, -0.0005636724387, 0.002786871125, -0.002457789311, 0.002077211093, -0.001609000794, 0.0009289570415, -0.002198313589, 0.001857914083, -0.00143913406, 0.0008308844372, 0.001609000794, -0.001246326656, 0.0007195670302, -0.001017621453, 0.0005875240199, 0.0004154422186, 0.007055935247, -0.006382333571, 0.005628689138, -0.004757110573, 0.003684842005, -0.002127444524, -0.005826246777, 0.005138266683, -0.004342627949, 0.003363785145, -0.001942082259, 0.004595805436, -0.003884164518, 0.003008660899, -0.00173705118, -0.003363785145, 0.002605576769, -0.001504330449, 0.002127444524, -0.001228280668, -0.0008685255898},
    {0.009450674049, 0, -0.007041226712, 0.002489449578, 0, 0, 0.009471262597, 0, -0.005155502352, 0.001822745337, 0, 0, 0, 0.004509379509, 0, 0, -0.002928927908, 0, 0.001594306415, -0.0005636724387, 0, 0, 0, 0, 0.004644785208, 0, 0, 0, -0.003323537749, 0, 0, 0.002158701091, 0, -0.00117504804, 0.0004154422186, 0, 0, 0, 0, 0, 0.01276466714, 0, 0, 0, 0, -0.009710411295, 0, 0, 0, 0.006948204719, 0, 0, -0.004512991348, 0, 0.002456561337, -0.0008685255898},
    {0.009450674049, 0, 0, -0.007468348734, 0, 0, 0, 0, 0, 0.01093647202, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.005636724387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.006231633279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.01823903739},
    {-0.01002344217, 0, 0.03747187483, -0.02896141836, 0, 0, 0.02841378779, 0, 0.005155502352, -0.006197334145, 0, 0, 0, 0.03156565657, 0, 0, -0.0113902752, 0, 0.002657177358, -0.0001878908129, 0, 0, 0, 0, 0.02322392604, 0, 0, 0, -0.001661768874, 0, 0, -0.006476103272, 0, 0.006462764219, -0.00290809553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02136290485, 0, 0, 0, -0.02779281887, 0, 0, 0.02437015328, 0, -0.01572199256, 0.006079679129},
    {0.01002344217, 0, -0.02875748534, 0.05360943398, 0, 0, -0.01894252519, 0, 0.01546650706, 0.01531106083, 0, 0, 0, -0.004509379509, 0, 0, 0.0370997535, 0, -0.02816608, 0.01089766715, 0, 0, 0, 0, 0, 0, 0, 0, 0.02991183974, 0, 0, -0.004317402181, 0, -0.01233800442, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03126692123, 0, 0, -0.04874030656, 0, 0.04127023046, -0.01823903739},
    {0.01002344217, 0, 0.04095763063, -0.04498262852, 0, 0, 0.01894252519, 0, -0.005155502352, -0.02114384591, 0, 0, 0, 0, 0, 0, 0.004556110079, 0, 0.0356061766, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03022181527, 0, 0.00411266814, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03790912732, 0, -0.05502697394, 0.03039839564},
    {-0.01002344217, 0, -0.01481446214, 0.04498262852, 0, 0, 0, 0, 0.02062200941, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, 0, 0.01062870943, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633407, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185872, -0.03039839564},
    {-0.01002344217, 0.03245159553, -0.01873593742, -0.02896141836, 0.021178387, -0.01640470803, 0.009471262597, 0.004464796006, -0.002577751176, -0.006197334145, 0.02087871931, -0.01764573846, 0.01366833024, -0.007891414141, -0.008489809874, 0.006576178451, -0.003796758399, 0.002301183095, -0.001328588679, -0.0001878908129, 0.01393435562, -0.01228894655, 0.01038605547, -0.00804500397, 0.004644785208, -0.001099156795, 0.0009289570415, -0.0007195670302, 0.0004154422186, -0.004827002382, 0.003738979967, -0.002158701091, 0.005596917993, -0.00323138211, -0.00290809553, 0, 0, 0, 0, 0, 0, 0.01281774291, -0.0113041867, 0.009553781488, -0.007400327319, 0.00427258097, -0.01838322174, 0.01553665807, -0.01203464359, 0.006948204719, 0.01816443978, -0.01407011456, 0.008123384426, -0.01361564495, 0.007860996278, 0.006079679129},
    {0.01002344217, -0.02490471285, 0.01437874267, 0.05360943398, -0.01411892467, 0.01093647202, -0.006314175065, 0.01339438802, -0.007733253528, 0.01531106083, -0.002982674187, 0.00252081978, -0.001952618605, 0.001127344877, 0.02765252359, -0.02141955267, 0.0123665845, -0.0243925408, 0.01408304, 0.01089766715, 0, 0, 0, 0, 0, 0.0197848223, -0.01672122675, 0.01295220654, -0.007477959935, -0.003218001588, 0.002492653312, -0.00143913406, -0.01068502526, 0.006169002209, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02068112446, -0.01747874033, 0.01353897404, -0.007816730308, -0.03632887957, 0.02814022911, -0.01624676885, 0.035741068, -0.02063511523, -0.01823903739},
    {0.01002344217, 0.0354703486, -0.02047881531, -0.04498262852, 0.01411892467, -0.01093647202, 0.006314175065, -0.004464796006, 0.002577751176, -0.02114384591, 0, 0, 0, 0, 0.00339592395, -0.00263047138, 0.00151870336, 0.03083585347, -0.0178030883, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02252601112, -0.01744857318, 0.01007393842, 0.003561675086, -0.00205633407, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02825579522, -0.02188684486, 0.01263637577, -0.04765475733, 0.02751348697, 0.03039839564},
    {-0.01002344217, -0.01282970056, 0.007407231071, 0.04498262852, 0, 0, 0, 0.01785918402, -0.0103110047, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, 0.009204732379, -0.005314354717, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.01780837543, -0.01028167035, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02978422333, -0.01719592936, -0.03039839564},
    {-0.01002344217, 0.03245159553, -0.03355039956, -0.008010605078, 0.021178387, 0.001093647202, -0.005682757558, -0.01696622482, 0.005671052587, 0.008384628549, 0.02087871931, -0.01092355238, 0.004990025324, -0.001878908129, -0.01625192176, 0.009582431457, -0.00466458889, 0.01089226665, -0.005580072453, -0.004697270322, 0.01393435562, -0.002457789311, -0.004154422186, 0.006436003176, -0.004644785208, -0.01209072474, 0.004644785208, -0.0007195670302, -0.0004154422186, 0.009654004764, -0.004985306623, 0.002158701091, -0.006614539446, 0.00323138211, 0.00290809553, 0, 0.01276466714, -0.0202632809, 0.02283413075, -0.02063511523, 0.01276466714, -0.001165249355, -0.007193573356, 0.01129083267, -0.01143686949, 0.007379912584, 0.001838322174, 0.003107331614, -0.004813857438, 0.003474102359, -0.002018271087, -0.0005211153539, 0.0009025982696, 0.001701955619, -0.0004913122674, -0.0008685255898},
    {0.01002344217, -0.02490471285, 0.0553363733, -0.004313402734, -0.01411892467, 0.01531106083, 0.008839845091, 0.008036632811, -0.01598205729, 0.0007290981347, -0.002982674187, 0.02772901758, -0.03015710957, 0.01916486291, -0.001455395978, -0.0116492304, 0.009763093026, 0.003528480745, 0.001328588679, -0.00263047138, 0, 0.01966231448, -0.007270238826, -0.007240503573, 0.009289570415, -0.002198313589, -0.01486331266, 0.01295220654, -0.006647075498, 0.004022501985, 0.006854796607, -0.005756536242, -0.004579296539, -0.00029376201, 0.00290809553, 0, 0, 0.0202632809, -0.03995972881, 0.04642900927, -0.03191166785, 0, -0.004110613346, -0.003039839564, 0.009082219892, -0.007768329036, 0.0004595805436, 0.00427258097, -0.002707794809, 0.0008685255898, -0.001009135544, -0.002345019093, 0.001805196539, 0.001276466714, 0.0002456561337, -0.0008685255898},
    {0.01002344217, 0.0354703486, -0.04923630065, -0.004313402734, 0.01411892467, -0.006561883212, -0.01641685517, -0.009822551213, 0.01495095682, 0.0007290981347, 0, 0.00336109304, 0.02820449096, -0.03532347282, -0.0004851319928, -0.01240079365, 0.008895262535, 0.00291483192, 0.002391459623, -0.00263047138, 0, 0, 0.02181071648, -0.004022501985, -0.009289570415, 0, -0.005573742249, -0.0143913406, 0.01412503543, 0.000804500397, 0.009347449919, -0.002878268121, -0.002544053633, -0.00381890613, 0.00290809553, 0, 0, 0, 0.02663981921, -0.05158778807, 0.04254889047, 0, 0, -0.009119518693, 0.008409462863, -0.003884164518, 0, 0.002330498711, 0.003008660899, -0.003474102359, -0.0003363785145, -0.002866134446, 0.001203464359, 0.0008509778095, 0.0009826245347, -0.0008685255898},
    {-0.01002344217, -0.01282970056, 0.0448791059, -0.008010605078, 0, 0.01749835523, 0.01325976764, -0.003571836805, -0.017528708, 0.008384628549, 0, 0, 0.008678304912, 0.02254689755, 0, -0.003006253006, -0.01920074962, 0.0006136488253, 0.01222301585, -0.004697270322, 0, 0, 0, 0.01609000794, 0.004644785208, 0, 0, -0.007195670302, -0.01080149768, 0, 0.002492653312, 0.01079350545, -0.0005088107266, -0.007344050249, 0.00290809553, 0, 0, 0, 0, 0.02579389404, -0.03191166785, 0, 0, 0, -0.01345514058, 0.01359457581, 0, 0, 0.006017321797, -0.003474102359, 0, -0.002084461416, -0.0009025982696, 0.0004254889047, 0.001719592936, -0.0008685255898},
    {-0.01002344217, -0.03245159553, -0.01873593742, -0.02896141836, 0.021178387, 0.01640470803, 0.009471262597, -0.004464796006, -0.002577751176, -0.006197334145, -0.02087871931, -0.01764573846, -0.01366833024, -0.007891414141, -0.008489809874, -0.006576178451, -0.003796758399, -0.002301183095, -0.001328588679, -0.0001878908129, 0.01393435562, 0.01228894655, 0.01038605547, 0.00804500397, 0.004644785208, 0.001099156795, 0.0009289570415, 0.0007195670302, 0.0004154422186, -0.004827002382, -0.003738979967, -0.002158701091, -0.005596917993, -0.00323138211, -0.00290809553, 0, 0, 0, 0, 0, 0, 0.01281774291, 0.0113041867, 0.009553781488, 0.007400327319, 0.00427258097, 0.01838322174, 0.01553665807, 0.01203464359, 0.006948204719, 0.01816443978, 0.01407011456, 0.008123384426, 0.01361564495, 0.007860996278, 0.006079679129},
    {0.01002344217, 0.02490471285, 0.01437874267, 0.05360943398, -0.01411892467, -0.01093647202, -0.006314175065, -0.01339438802, -0.007733253528, 0.01531106083, 0.002982674187, 0.00252081978, 0.001952618605, 0.001127344877, 0.02765252359, 0.02141955267, 0.0123665845, 0.0243925408, 0.01408304, 0.01089766715, 0, 0, 0, 0, 0, -0.0197848223, -0.01672122675, -0.01295220654, -0.007477959935, -0.003218001588, -0.002492653312, -0.00143913406, 0.01068502526, 0.006169002209, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.02068112446, -0.01747874033, -0.01353897404, -0.007816730308, -0.03632887957, -0.02814022911, -0.01624676885, -0.035741068, -0.02063511523, -0.01823903739},
    {0.01002344217, -0.0354703486, -0.02047881531, -0.04498262852, 0.01411892467, 0.01093647202, 0.006314175065, 0.004464796006, 0.002577751176, -0.02114384591, 0, 0, 0, 0, 0.00339592395, 0.00263047138, 0.00151870336, -0.03083585347, -0.0178030883, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02252601112, 0.01744857318, 0.01007393842, -0.003561675086, -0.00205633407, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02825579522, 0.02188684486, 0.01263637577, 0.04765475733, 0.02751348697, 0.03039839564},
    {-0.01002344217, 0.01282970056, 0.007407231071, 0.04498262852, 0, 0, 0, -0.01785918402, -0.0103110047, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, -0.009204732379, -0.005314354717, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.01780837543, -0.01028167035, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.02978422333, -0.01719592936, -0.03039839564},
    {-0.01002344217, -0.03245159553, -0.03355039956, -0.008010605078, 0.021178387, -0.001093647202, -0.005682757558, 0.01696622482, 0.005671052587, 0.008384628549, -0.02087871931, -0.01092355238, -0.004990025324, -0.001878908129, -0.01625192176, -0.009582431457, -0.00466458889, -0.01089226665, -0.005580072453, -0.004697270322, 0.01393435562, 0.002457789311, -0.004154422186, -0.006436003176, -0.004644785208, 0.01209072474, 0.004644785208, 0.0007195670302, -0.0004154422186, 0.009654004764, 0.004985306623, 0.002158701091, 0.006614539446, 0.00323138211, 0.00290809553, 0, 0.01276466714, 0.0202632809, 0.02283413075, 0.02063511523, 0.01276466714, -0.001165249355, 0.007193573356, 0.01129083267, 0.01143686949, 0.007379912584, -0.001838322174, 0.003107331614, 0.004813857438, 0.003474102359, -0.002018271087, 0.0005211153539, 0.0009025982696, -0.001701955619, -0.0004913122674, -0.0008685255898},
    {0.01002344217, 0.02490471285, 0.0553363733, -0.004313402734, -0.01411892467, -0.01531106083, 0.008839845091, -0.008036632811, -0.01598205729, 0.0007290981347, 0.002982674187, 0.02772901758, 0.03015710957, 0.01916486291, -0.001455395978, 0.0116492304, 0.009763093026, -0.003528480745, 0.001328588679, -0.00263047138, 0, -0.01966231448, -0.007270238826, 0.007240503573, 0.009289570415, 0.002198313589, -0.01486331266, -0.01295220654, -0.006647075498, 0.004022501985, -0.006854796607, -0.005756536242, 0.004579296539, -0.00029376201, 0.00290809553, 0, 0, -0.0202632809, -0.03995972881, -0.04642900927, -0.03191166785, 0, 0.004110613346, -0.003039839564, -0.009082219892, -0.007768329036, -0.0004595805436, 0.00427258097, 0.002707794809, 0.0008685255898, -0.001009135544, 0.002345019093, 0.001805196539, -0.001276466714, 0.0002456561337, -0.0008685255898},
    {0.01002344217, -0.0354703486, -0.04923630065, -0.004313402734, 0.01411892467, 0.006561883212, -0.01641685517, 0.009822551213, 0.01495095682, 0.0007290981347, 0, 0.00336109304, -0.02820449096, -0.03532347282, -0.0004851319928, 0.01240079365, 0.008895262535, -0.00291483192, 0.002391459623, -0.00263047138, 0, 0, 0.02181071648, 0.004022501985, -0.009289570415, 0, -0.005573742249, 0.0143913406, 0.01412503543, 0.000804500397, -0.009347449919, -0.002878268121, 0.002544053633, -0.00381890613, 0.00290809553, 0, 0, 0, 0.02663981921, 0.05158778807, 0.04254889047, 0, 0, -0.009119518693, -0.008409462863, -0.003884164518, 0, 0.002330498711, -0.003008660899, -0.003474102359, -0.0003363785145, 0.002866134446, 0.001203464359, -0.0008509778095, 0.0009826245347, -0.0008685255898},
    {-0.01002344217, 0.01282970056, 0.0448791059, -0.008010605078, 0, -0.01749835523, 0.01325976764, 0.003571836805, -0.017528708, 0.008384628549, 0, 0, -0.008678304912, 0.02254689755, 0, 0.003006253006, -0.01920074962, -0.0006136488253, 0.01222301585, -0.004697270322, 0, 0, 0, -0.01609000794, 0.004644785208, 0, 0, 0.007195670302, -0.01080149768, 0, -0.002492653312, 0.01079350545, 0.0005088107266, -0.007344050249, 0.00290809553, 0, 0, 0, 0, -0.02579389404, -0.03191166785, 0, 0, 0, 0.01345514058, 0.01359457581, 0, 0, -0.006017321797, -0.003474102359, 0, 0.002084461416, -0.0009025982696, -0.0004254889047, 0.001719592936, -0.0008685255898},
    {-0.01002344217, -0.04528129609, -0.01132870634, -0.008010605078, -0.001411892467, 0.01640470803, 0.01452260265, 0.01339438802, 0.01185765541, 0.008384628549, -0.008948022562, -0.01428464542, -0.01453616073, -0.009394540645, -0.01237086582, -0.01258868446, -0.008135910855, -0.01027861782, -0.006642943396, -0.004697270322, -0.008360613374, 0.002457789311, 0.008308844372, 0.009654004764, 0.006502699291, 0.002198313589, 0.007431656332, 0.008634804363, 0.005816191061, 0.006436003176, 0.007477959935, 0.005036969212, 0.006105728719, 0.00411266814, 0.00290809553, 0.03527967624, 0.01914700071, 0.007880164793, 0.0009514221146, -0.002210905203, -0.002127444524, 0.01747874033, 0.007193573356, 0.0008685255898, -0.002018271087, -0.001942082259, 0.00643412761, 0.0007768329036, -0.001805196539, -0.00173705118, 0.000672757029, -0.001563346062, -0.001504330449, -0.001276466714, -0.001228280668, -0.0008685255898},
    {0.01002344217, 0.06037506145, -0.006100072647, -0.004313402734, -0.005647569867, -0.02187294404, 0.001262835013, -0.01785918402, 0.00103110047, 0.0007290981347, 0.04175743862, 0.0168054652, -0.0008678304912, -0.005260942761, 0.01455395978, -0.0007515632516, -0.004556110079, -0.0006136488253, -0.003720048302, -0.00263047138, 0.005573742249, -0.01966231448, -0.01038605547, 0.003218001588, 0.006502699291, -0.01758650871, -0.009289570415, 0.002878268121, 0.005816191061, -0.00804500397, 0.002492653312, 0.005036969212, 0.002035242906, 0.00411266814, 0.00290809553, -0.07055935247, -0.01276466714, 0.00450295131, 0.003805688459, -0.000736968401, -0.002127444524, -0.01165249355, 0.004110613346, 0.003474102359, -0.000672757029, -0.001942082259, 0.003676644349, 0.003107331614, -0.0006017321797, -0.00173705118, 0.002691028116, -0.0005211153539, -0.001504330449, -0.0004254889047, -0.001228280668, -0.0008685255898},
    {0.01002344217, -0.06037506145, -0.006100072647, -0.004313402734, -0.005647569867, 0.02187294404, 0.001262835013, 0.01785918402, 0.00103110047, 0.0007290981347, -0.04175743862, 0.0168054652, 0.0008678304912, -0.005260942761, 0.01455395978, 0.0007515632516, -0.004556110079, 0.0006136488253, -0.003720048302, -0.00263047138, 0.005573742249, 0.01966231448, -0.01038605547, -0.003218001588, 0.006502699291, 0.01758650871, -0.009289570415, -0.002878268121, 0.005816191061, -0.00804500397, -0.002492653312, 0.005036969212, -0.002035242906, 0.00411266814, 0.00290809553, 0.07055935247, -0.01276466714, -0.00450295131, 0.003805688459, 0.000736968401, -0.002127444524, -0.01165249355, -0.004110613346, 0.003474102359, 0.000672757029, -0.001942082259, -0.003676644349, 0.003107331614, 0.0006017321797, -0.00173705118, 0.002691028116, 0.0005211153539, -0.001504330449, 0.0004254889047, -0.001228280668, -0.0008685255898},
    {-0.01002344217, 0.04528129609, -0.01132870634, -0.008010605078, -0.001411892467, -0.01640470803, 0.01452260265, -0.01339438802, 0.01185765541, 0.008384628549, 0.008948022562, -0.01428464542, 0.01453616073, -0.009394540645, -0.01237086582, 0.01258868446, -0.008135910855, 0.01027861782, -0.006642943396, -0.004697270322, -0.008360613374, -0.002457789311, 0.008308844372, -0.009654004764, 0.006502699291, -0.002198313589, 0.007431656332, -0.008634804363, 0.005816191061, 0.006436003176, -0.007477959935, 0.005036969212, -0.006105728719, 0.00411266814, 0.00290809553, -0.03527967624, 0.01914700071, -0.007880164793, 0.0009514221146, 0.002210905203, -0.002127444524, 0.01747874033, -0.007193573356, 0.0008685255898, 0.002018271087, -0.001942082259, -0.00643412761, 0.0007768329036, 0.001805196539, -0.00173705118, 0.000672757029, 0.001563346062, -0.001504330449, 0.001276466714, -0.001228280668, -0.0008685255898},
    {0.07875561708, 0.06792194414, -0.004357194748, -0.003081001953, 0.07059462333, -0.03280941606, 0.01262835013, -0.02678877604, 0.03608851646, -0.007290981347, 0.02982674187, 0.01512491868, -0.03254364342, 0.0248015873, 0.01309856381, -0.02292267917, 0.01692269458, -0.02485277742, 0.01647449962, 0.01427970178, 0, 0.02457789311, -0.03635119413, 0.03620251786, -0.02322392604, 0.02198313589, -0.007431656332, 0, 0.001661768874, -0.03459351707, 0.01308642989, -0.003597835151, 0.03358150796, -0.0146881005, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0.02260837341, -0.03343823521, 0.03330147294, -0.02136290485, -0.004595805436, -0.01942082259, 0.02707794809, -0.01910756298, 0.009082219892, 0.007035057278, -0.008123384426, -0.01021173371, 0.001965249069, 0.006079679129},
    {-0.02147880466, 0.003773441341, 0.002178597374, -0.02156701367, 0, 0.09842824818, -0.08524136337, -0.03125357204, -0.01804425823, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.01164316783, 0.02367424242, -0.03351995272, -0.001380709857, -0.0007971532075, 0.003945707071, 0, 0, 0.03635119413, -0.06033752977, 0.04644785208, 0, 0.02415288308, 0, -0.01080149768, -0.007240503573, -0.03925928966, 0.02590441309, 0.01882599688, 0.01086919437, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0.03343823521, -0.0555024549, 0.0427258097, 0, -0.01553665807, 0, 0.006948204719, 0.003027406631, 0.01641513365, -0.01083117923, -0.006807822476, -0.003930498139, 0.006079679129},
    {0.07875561708, 0.03018753073, 0.06100072647, -0.003081001953, 0, 0.02187294404, 0.07577010078, 0.01785918402, -0.04124401881, -0.007290981347, 0, 0, 0.05206982947, -0.009018759019, 0, -0.01277657528, 0.02863840621, 0.001840946476, -0.02976038641, 0.01427970178, 0, 0, 0, 0.04022501985, -0.04644785208, 0, 0, 0.0143913406, 0.01827945762, 0, -0.01370959321, -0.03453921745, 0.004070485813, 0.03642648924, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016366, -0.0427258097, 0, 0, -0.03008660899, 0.02431871652, 0, 0.01407011456, 0, -0.003403911238, -0.009826245347, 0.006079679129},
    {-0.02147880466, 0.003773441341, -0.01960737637, 0.009243005859, 0, -0.01093647202, 0.009471262597, 0.1026903081, -0.06959928175, -0.06015059611, 0, -0.00504163956, 0.006508728684, -0.004509379509, 0.04657267131, 0.02141955267, -0.02896384264, -0.007824022522, -0.02630605585, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0.0334424535, -0.04317402181, 0.02991183974, 0.02896201429, -0.002492653312, -0.004317402181, -0.06054847647, 0.01850700663, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03495748066, -0.04512991348, 0.03126692123, -0.01210962652, -0.02814022911, 0.02707794809, 0.02382737866, 0, -0.01823903739},
    {-0.02147880466, -0.01509376536, 0.01307158424, 0.009243005859, 0, -0.01093647202, 0.009471262597, -0.008929592012, 0.1237320564, -0.06015059611, 0, 0, -0.006508728684, 0.006764069264, 0, 0.05749458874, 0.01269202093, -0.0266937239, 0.00637722566, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0, 0.04317402181, -0.04486775961, 0, 0.01994122649, 0.02158701091, -0.01424670034, -0.06169002209, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.04512991348, -0.04690038185, 0, -0.03752030548, 0.01624676885, 0.01191368933, 0.02063511523, -0.01823903739},
    {0.07875561708, 0.03018753073, 0.01742877899, 0.05853903711, 0, 0.02187294404, -0.01894252519, 0.01785918402, 0.0103110047, 0.08384628549, 0, 0, 0, 0, 0, 0.005260942761, -0.004556110079, 0.0533874478, 0.03082325736, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03489714636, -0.03022181527, 0.02849340069, 0.01645067256, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.04377368973, -0.03790912732, -0.02382737866, -0.01375674349, 0.03039839564},
    {0.07875561708, -0.06792194414, -0.004357194748, -0.003081001953, 0.07059462333, 0.03280941606, 0.01262835013, 0.02678877604, 0.03608851646, -0.007290981347, -0.02982674187, 0.01512491868, 0.03254364342, 0.0248015873, 0.01309856381, 0.02292267917, 0.01692269458, 0.02485277742, 0.01647449962, 0.01427970178, 0, -0.02457789311, -0.03635119413, -0.03620251786, -0.02322392604, -0.02198313589, -0.007431656332, 0, 0.001661768874, -0.03459351707, -0.01308642989, -0.003597835151, -0.03358150796, -0.0146881005, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, -0.02260837341, -0.03343823521, -0.03330147294, -0.02136290485, 0.004595805436, -0.01942082259, -0.02707794809, -0.01910756298, 0.009082219892, -0.007035057278, -0.008123384426, 0.01021173371, 0.001965249069, 0.006079679129},
    {-0.02147880466, -0.003773441341, 0.002178597374, -0.02156701367, 0, -0.09842824818, -0.08524136337, 0.03125357204, -0.01804425823, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.01164316783, -0.02367424242, -0.03351995272, 0.001380709857, -0.0007971532075, 0.003945707071, 0, 0, 0.03635119413, 0.06033752977, 0.04644785208, 0, 0.02415288308, 0, -0.01080149768, -0.007240503573, 0.03925928966, 0.02590441309, -0.01882599688, 0.01086919437, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0.03343823521, 0.0555024549, 0.0427258097, 0, -0.01553665807, 0, 0.006948204719, 0.003027406631, -0.01641513365, -0.01083117923, 0.006807822476, -0.003930498139, 0.006079679129},
    {0.07875561708, -0.03018753073, 0.06100072647, -0.003081001953, 0, -0.02187294404, 0.07577010078, -0.01785918402, -0.04124401881, -0.007290981347, 0, 0, -0.05206982947, -0.009018759019, 0, 0.01277657528, 0.02863840621, -0.001840946476, -0.02976038641, 0.01427970178, 0, 0, 0, -0.04022501985, -0.04644785208, 0, 0, -0.0143913406, 0.01827945762, 0, 0.01370959321, -0.03453921745, -0.004070485813, 0.03642648924, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016366, -0.0427258097, 0, 0, 0.03008660899, 0.02431871652, 0, -0.01407011456, 0, 0.003403911238, -0.009826245347, 0.006079679129},
    {-0.02147880466, -0.003773441341, -0.01960737637, 0.009243005859, 0, 0.01093647202, 0.009471262597, -0.1026903081, -0.06959928175, -0.06015059611, 0, -0.00504163956, -0.006508728684, -0.004509379509, 0.04657267131, -0.02141955267, -0.02896384264, 0.007824022522, -0.02630605585, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0.0334424535, 0.04317402181, 0.02991183974, 0.02896201429, 0.002492653312, -0.004317402181, 0.06054847647, 0.01850700663, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03495748066, 0.04512991348, 0.03126692123, -0.01210962652, 0.02814022911, 0.02707794809, -0.02382737866, 0, -0.01823903739},
    {-0.02147880466, 0.01509376536, 0.01307158424, 0.009243005859, 0, 0.01093647202, 0.009471262597, 0.008929592012, 0.1237320564, -0.06015059611, 0, 0, 0.006508728684, 0.006764069264, 0, -0.05749458874, 0.01269202093, 0.0266937239, 0.00637722566, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0, -0.04317402181, -0.04486775961, 0, -0.01994122649, 0.02158701091, 0.01424670034, -0.06169002209, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04512991348, -0.04690038185, 0, 0.03752030548, 0.01624676885, -0.01191368933, 0.02063511523, -0.01823903739},
    {0.07875561708, -0.03018753073, 0.01742877899, 0.05853903711, 0, -0.02187294404, -0.01894252519, -0.01785918402, 0.0103110047, 0.08384628549, 0, 0, 0, 0, 0, -0.005260942761, -0.004556110079, -0.0533874478, 0.03082325736, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.03489714636, -0.03022181527, -0.02849340069, 0.01645067256, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04377368973, -0.03790912732, 0.02382737866, -0.01375674349, 0.03039839564},
    {0.07875561708, -0.03773441341, -0.05664353172, -0.003081001953, 0.042356774, 0.0546823601, 0.03788505039, 0.04464796006, 0.005155502352, -0.007290981347, 0.04175743862, -0.00504163956, -0.024733169, -0.02029220779, 0.02959305156, 0.0101461039, 0.002169576228, 0.0266937239, 0.01328588679, 0.01427970178, -0.05573742249, -0.02457789311, -0.005193027733, 0.004022501985, 0.004644785208, -0.002198313589, -0.01300539858, -0.01583047467, -0.01080149768, -0.01689450834, -0.0267960231, -0.01942830982, -0.02951102214, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, -0.05127097164, -0.02260837341, -0.004776890744, 0.00370016366, 0.00427258097, -0.03676644349, -0.007768329036, 0.006017321797, 0.006948204719, -0.009082219892, 0.007035057278, 0.008123384426, 0.006807822476, 0.007860996278, 0.006079679129},
    {-0.02147880466, 0, -0.004357194748, -0.02156701367, -0.127070322, 0, 0.02841378779, 0, 0.03608851646, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.04220648338, 0, -0.00618329225, 0, 0.001594306415, 0.003945707071, 0.08360613374, 0, -0.01038605547, 0, 0.004644785208, 0, 0.02415288308, 0, -0.01080149768, 0.04344302144, 0, -0.01942830982, 0, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, 0.07690645746, 0, -0.009553781488, 0, 0.00427258097, 0, -0.01553665807, 0, 0.006948204719, -0.01816443978, 0, 0.008123384426, 0, 0.007860996278, 0.006079679129},
    {0.07875561708, 0.03773441341, -0.05664353172, -0.003081001953, 0.042356774, -0.0546823601, 0.03788505039, -0.04464796006, 0.005155502352, -0.007290981347, -0.04175743862, -0.00504163956, 0.024733169, -0.02029220779, 0.02959305156, -0.0101461039, 0.002169576228, -0.0266937239, 0.01328588679, 0.01427970178, -0.05573742249, 0.02457789311, -0.005193027733, -0.004022501985, 0.004644785208, 0.002198313589, -0.01300539858, 0.01583047467, -0.01080149768, -0.01689450834, 0.0267960231, -0.01942830982, 0.02951102214, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, -0.05127097164, 0.02260837341, -0.004776890744, -0.00370016366, 0.00427258097, 0.03676644349, -0.007768329036, -0.006017321797, 0.006948204719, -0.009082219892, -0.007035057278, 0.008123384426, -0.006807822476, 0.007860996278, 0.006079679129},
    {-0.02147880466, -0.0188672067, 0.006535792122, 0.009243005859, 0.01411892467, 0, -0.003157087532, -0.1116199001, -0.05413277469, -0.06015059611, -0.008948022562, -0.00252081978, 0.0006508728684, 0.001127344877, -0.02765252359, 0.03607503608, 0.03742518993, -0.01886970138, 0.01992883019, 0.0005636724387, 0, 0, 0, 0, 0, 0.05935446691, 0.01672122675, -0.004317402181, -0.007477959935, 0.003218001588, 0.0224338798, 0.01870874279, 0.04630177612, 0.04318301547, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.06204337339, 0.01747874033, -0.004512991348, -0.007816730308, 0.03632887957, -0.00938007637, -0.01624676885, -0.01191368933, -0.02063511523, -0.01823903739},
    {-0.02147880466, 0.0188672067, 0.006535792122, 0.009243005859, 0.01411892467, 0, -0.003157087532, 0.1116199001, -0.05413277469, -0.06015059611, 0.008948022562, -0.00252081978, -0.0006508728684, 0.001127344877, -0.02765252359, -0.03607503608, 0.03742518993, 0.01886970138, 0.01992883019, 0.0005636724387, 0, 0, 0, 0, 0, -0.05935446691, 0.01672122675, 0.004317402181, -0.007477959935, 0.003218001588, -0.0224338798, 0.01870874279, -0.04630177612, 0.04318301547, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.06204337339, 0.01747874033, 0.004512991348, -0.007816730308, 0.03632887957, 0.00938007637, -0.01624676885, 0.01191368933, -0.02063511523, -0.01823903739},
    {0.07875561708, 0, -0.03485755798, 0.05853903711, -0.02823784933, 0, 0.006314175065, 0, -0.02062200941, 0.08384628549, 0, 0, 0, 0, -0.006791847899, 0, 0.00151870336, 0, -0.06164651471, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04505202223, 0, 0.01007393842, 0, -0.03290134512, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.05651159044, 0, 0.01263637577, 0, 0.02751348697, 0.03039839564},
    {0.07875561708, -0.03773441341, -0.02178597374, -0.0523770332, 0.042356774, 0.0546823601, 0, 0.04464796006, 0.02577751176, 0.02916392539, 0.04175743862, 0.02856929084, 0.02733666047, 0.01878908129, -0.009217507864, -0.007891414141, -0.003471321965, -0.02485277742, -0.01434875774, -0.01503126503, -0.05573742249, -0.004915578621, -0.01973350538, -0.03942051945, -0.03251349645, -0.02418144948, -0.009289570415, -0.01583047467, -0.01412503543, -0.002413501191, -0.0006231633279, -0.002158701091, 0.007123350172, 0.00411266814, 0.005400748842, 0, -0.05105866857, -0.04052656179, -0.01141706538, 0.01031755761, 0.01276466714, 0.004660997422, -0.01438714671, -0.005645416334, 0.005718434747, 0.007379912584, 0.003676644349, -0.001553665807, 0.002406928719, 0.003474102359, 0.001009135544, 0.0002605576769, 0.0009025982696, -0.0008509778095, -0.0004913122674, -0.0008685255898},
    {-0.02147880466, 0, -0.02178597374, 0.003081001953, -0.127070322, 0, 0.04735631299, 0, 0.02577751176, 0.01275921736, 0, -0.03529147692, 0, 0.002254689755, 0.05093885925, 0, -0.009437656592, 0, -0.01434875774, -0.01296446609, 0.08360613374, 0, 0.04777585514, 0, -0.03251349645, 0, 0.009289570415, 0, -0.01412503543, -0.01448100715, 0, -0.002158701091, 0, 0.00411266814, 0.005400748842, 0, 0.07658800285, 0, -0.02283413075, 0, 0.01276466714, -0.006991496133, 0, -0.01129083267, 0, 0.007379912584, 0, -0.003107331614, 0, 0.003474102359, 0.002018271087, 0, 0.0009025982696, 0, -0.0004913122674, -0.0008685255898},
    {0.07875561708, 0.03773441341, -0.02178597374, -0.0523770332, 0.042356774, -0.0546823601, 0, -0.04464796006, 0.02577751176, 0.02916392539, -0.04175743862, 0.02856929084, -0.02733666047, 0.01878908129, -0.009217507864, 0.007891414141, -0.003471321965, 0.02485277742, -0.01434875774, -0.01503126503, -0.05573742249, 0.004915578621, -0.01973350538, 0.03942051945, -0.03251349645, 0.02418144948, -0.009289570415, 0.01583047467, -0.01412503543, -0.002413501191, 0.0006231633279, -0.002158701091, -0.007123350172, 0.00411266814, 0.005400748842, 0, -0.05105866857, 0.04052656179, -0.01141706538, -0.01031755761, 0.01276466714, 0.004660997422, 0.01438714671, -0.005645416334, -0.005718434747, 0.007379912584, -0.003676644349, -0.001553665807, -0.002406928719, 0.003474102359, 0.001009135544, -0.0002605576769, 0.0009025982696, 0.0008509778095, -0.0004913122674, -0.0008685255898},
    {-0.02147880466, -0.0188672067, 0.01089298687, 0.003081001953, 0.01411892467, -0.1093647202, -0.07892718831, 0.02232398003, -0.01288875588, 0.01275921736, -0.008948022562, -0.02772901758, -0.005857855816, 0.01916486291, 0.001455395978, 0.03832972583, 0.03482169846, -0.01242638871, 0.007174378868, -0.01296446609, 0, 0.05898694345, 0.007270238826, 0.05068352501, 0.06502699291, -0.006594940767, 0.01486331266, -0.004317402181, -0.001661768874, -0.004022501985, -0.008101123263, -0.01151307248, 0.003561675086, -0.00205633407, 0.005400748842, 0, 0, 0.06078984269, 0.03995972881, -0.01547633642, -0.03191166785, 0, -0.01233184004, 0.003039839564, -0.003027406631, -0.007768329036, 0.001378741631, -0.00427258097, 0.0009025982696, 0.0008685255898, 0.001009135544, 0.0007816730308, 0.001805196539, -0.0004254889047, 0.0002456561337, -0.0008685255898},
    {-0.02147880466, 0.0188672067, 0.01089298687, 0.003081001953, 0.01411892467, 0.1093647202, -0.07892718831, -0.02232398003, -0.01288875588, 0.01275921736, 0.008948022562, -0.02772901758, 0.005857855816, 0.01916486291, 0.001455395978, -0.03832972583, 0.03482169846, 0.01242638871, 0.007174378868, -0.01296446609, 0, -0.05898694345, 0.007270238826, -0.05068352501, 0.06502699291, 0.006594940767, 0.01486331266, 0.004317402181, -0.001661768874, -0.004022501985, 0.008101123263, -0.01151307248, -0.003561675086, -0.00205633407, 0.005400748842, 0, 0, -0.06078984269, 0.03995972881, 0.01547633642, -0.03191166785, 0, 0.01233184004, 0.003039839564, 0.003027406631, -0.007768329036, -0.001378741631, -0.00427258097, -0.0009025982696, 0.0008685255898, 0.001009135544, -0.0007816730308, 0.001805196539, 0.0004254889047, 0.0002456561337, -0.0008685255898},
    {0.07875561708, 0, 0.04357194748, -0.0523770332, -0.02823784933, 0, 0.06314175065, 0, -0.05155502352, 0.02916392539, 0, -0.00672218608, 0, -0.06012506013, 0.0009702639856, 0, -0.01258354212, 0, 0.02869751547, -0.01503126503, 0, 0, -0.04362143295, 0, -0.06502699291, 0, 0.0111474845, 0, 0.03157360861, -0.001609000794, 0, -0.002878268121, 0, -0.008225336279, 0.005400748842, 0, 0, 0, -0.05327963842, 0, 0.04254889047, 0, 0, 0.01823903739, 0, -0.003884164518, 0, -0.004660997422, 0, -0.003474102359, 0.000672757029, 0, 0.001203464359, 0, 0.0009826245347, -0.0008685255898},
    {0.1073940233, -0.188672067, -0.1089298687, -0.07702504883, -0.07059462333, -0.0546823601, -0.03157087532, -0.04464796006, -0.02577751176, -0.01822745337, 0.08948022562, -0.01512491868, 0.05857855816, 0.0744047619, -0.01309856381, -0.02818362193, 0.009763093026, 0.06903549284, 0.03985766038, 0.0597492785, 0, 0.07373367932, 0.03635119413, -0.01206750595, -0.02322392604, 0.06594940767, 0.007431656332, 0.04317402181, 0.04652952848, 0.03459351707, 0.0155790832, 0.02518484606, -0.02544053633, -0.0146881005, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, 0.06782512022, 0.03343823521, -0.01110049098, -0.02136290485, -0.01378741631, 0.01942082259, -0.009025982696, -0.01910756298, -0.009082219892, -0.002345019093, -0.008123384426, 0.003403911238, 0.001965249069, 0.006079679129},
    {0.1073940233, 0.188672067, -0.1089298687, -0.07702504883, -0.07059462333, 0.0546823601, -0.03157087532, 0.04464796006, -0.02577751176, -0.01822745337, -0.08948022562, -0.01512491868, -0.05857855816, 0.0744047619, -0.01309856381, 0.02818362193, 0.009763093026, -0.06903549284, 0.03985766038, 0.0597492785, 0, -0.07373367932, 0.03635119413, 0.01206750595, -0.02322392604, -0.06594940767, 0.007431656332, -0.04317402181, 0.04652952848, 0.03459351707, -0.0155790832, 0.02518484606, 0.02544053633, -0.0146881005, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, -0.06782512022, 0.03343823521, 0.01110049098, -0.02136290485, 0.01378741631, 0.01942082259, 0.009025982696, -0.01910756298, -0.009082219892, 0.002345019093, -0.008123384426, -0.003403911238, 0.001965249069, 0.006079679129},
    {0.1073940233, 0, 0.2178597374, -0.07702504883, 0, 0, -0.09471262597, 0, 0.05155502352, -0.01822745337, 0, -0.09074951207, 0, -0.0946969697, 0.02328633566, 0, -0.02278055039, 0, -0.07971532075, 0.0597492785, 0, 0, -0.07270238826, 0, 0.04644785208, 0, -0.04830576616, 0, -0.0781031371, 0.01448100715, 0, 0.04317402181, 0, 0.029376201, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, 0, -0.06687647042, 0, 0.0427258097, 0, 0.03107331614, 0, 0.006948204719, -0.006054813261, 0, -0.01083117923, 0, -0.003930498139, 0.006079679129},
    {0.1073940233, 0, 0, 0.2310751465, 0, 0, 0, 0, 0, -0.1093647202, 0, 0.01008327912, 0, -0.004509379509, -0.09314534262, 0, -0.1041396589, 0, 0, -0.09244227994, 0, 0, 0, 0, 0, 0, -0.06688490699, 0, 0.02991183974, -0.05792402858, 0, -0.06476103272, 0, 0, 0.08350388594, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.06991496133, 0, 0.03126692123, 0.02421925305, 0, 0.02707794809, 0, 0, -0.01823903739}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute constants
    const double C0 = element_coordinates[3][0] + element_coordinates[2][0] \
                    + element_coordinates[1][0] - element_coordinates[0][0];
    const double C1 = element_coordinates[3][1] + element_coordinates[2][1] \
                    + element_coordinates[1][1] - element_coordinates[0][1];
    const double C2 = element_coordinates[3][2] + element_coordinates[2][2] \
                    + element_coordinates[1][2] - element_coordinates[0][2];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = coordinates[0];
    double y = coordinates[1];
    double z = coordinates[2];
    
    x = (2.0*d00*x + 2.0*d10*y + 2.0*d20*z - d00*C0 - d10*C1 - d20*C2) / detJ;
    y = (2.0*d01*x + 2.0*d11*y + 2.0*d21*z - d01*C0 - d11*C1 - d21*C2) / detJ;
    z = (2.0*d02*x + 2.0*d12*y + 2.0*d22*z - d02*C0 - d12*C1 - d22*C2) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * (1.0 + x)/(y + z) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * (1.0 + y)/(1.0 - z) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[3][3] ={{2*d00 / detJ, 2*d10 / detJ, 2*d20 / detJ}, {2*d01 / detJ, 2*d11 / detJ, 2*d21 / detJ}, {2*d02 / detJ, 2*d12 / detJ, 2*d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.666666667*x*psitilde_a_2 - 0.6666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.02857142857*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.7714285714*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.01851851852*psitilde_bs_0_4 + 1.833333333*y*psitilde_bs_0_4 - 0.8148148148*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.2857142857*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.7142857143*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.1785714286*psitilde_bs_1_3 + 1.964285714*y*psitilde_bs_1_3 - 0.7857142857*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.020408163*psitilde_bs_2_1 + 2.571428571*y*psitilde_bs_2_1 - 0.5510204082*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.5787037037*psitilde_bs_2_2 + 2.291666667*y*psitilde_bs_2_2 - 0.712962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.512345679*psitilde_bs_3_1 + 3.055555556*y*psitilde_bs_3_1 - 0.5432098765*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.1555555556*psitilde_cs_00_2 + 1.866666667*z*psitilde_cs_00_2 - 0.7111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.06285714286*psitilde_cs_00_4 + 1.885714286*z*psitilde_cs_00_4 - 0.8228571429*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.7777777778*psitilde_cs_01_1 + 2.333333333*z*psitilde_cs_01_1 - 0.5555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.4285714286*psitilde_cs_01_2 + 2.142857143*z*psitilde_cs_01_2 - 0.7142857143*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.7333333333*psitilde_cs_02_2 + 2.444444444*z*psitilde_cs_02_2 - 0.7111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.7777777778*psitilde_cs_10_1 + 2.333333333*z*psitilde_cs_10_1 - 0.5555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.4285714286*psitilde_cs_10_2 + 2.142857143*z*psitilde_cs_10_2 - 0.7142857143*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.7333333333*psitilde_cs_11_2 + 2.444444444*z*psitilde_cs_11_2 - 0.7111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.7333333333*psitilde_cs_20_2 + 2.444444444*z*psitilde_cs_20_2 - 0.7111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.937253933*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.708203932*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.196152423*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.809475019*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.598076211*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.674234614*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.121320344*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.12429773*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.810708435*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.291561976*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.422616289*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.708099244*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.774964387*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.416198487*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.744562647*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.31662479*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.422616289*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.974937186*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.872281323*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.062019202*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520788*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.658312395*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.64581852*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476413*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.68332145*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.874208829*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.415880433*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.09338662*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.013878189*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.982120022*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.031128874*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.539392014*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.062257748*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.244997998*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.605551275*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.982120022*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.408326913*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.122498999*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.415880433*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.549509757*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.802775638*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    const static double coefficients0[56][56] = \
    {{0.009450674049, 0.006097881207, 0.003520613356, 0.002489449578, 0.007059462333, 0.00546823601, 0.003157087532, 0.004464796006, 0.002577751176, 0.001822745337, -0.002982674187, -0.00252081978, -0.001952618605, -0.001127344877, -0.002183093968, -0.001691017316, -0.0009763093026, -0.001380709857, -0.0007971532075, -0.0005636724387, 0.002786871125, 0.002457789311, 0.002077211093, 0.001609000794, 0.0009289570415, 0.002198313589, 0.001857914083, 0.00143913406, 0.0008308844372, 0.001609000794, 0.001246326656, 0.0007195670302, 0.001017621453, 0.0005875240199, 0.0004154422186, -0.007055935247, -0.006382333571, -0.005628689138, -0.004757110573, -0.003684842005, -0.002127444524, -0.005826246777, -0.005138266683, -0.004342627949, -0.003363785145, -0.001942082259, -0.004595805436, -0.003884164518, -0.003008660899, -0.00173705118, -0.003363785145, -0.002605576769, -0.001504330449, -0.002127444524, -0.001228280668, -0.0008685255898},
    {0.009450674049, -0.006097881207, 0.003520613356, 0.002489449578, 0.007059462333, -0.00546823601, 0.003157087532, -0.004464796006, 0.002577751176, 0.001822745337, 0.002982674187, -0.00252081978, 0.001952618605, -0.001127344877, -0.002183093968, 0.001691017316, -0.0009763093026, 0.001380709857, -0.0007971532075, -0.0005636724387, 0.002786871125, -0.002457789311, 0.002077211093, -0.001609000794, 0.0009289570415, -0.002198313589, 0.001857914083, -0.00143913406, 0.0008308844372, 0.001609000794, -0.001246326656, 0.0007195670302, -0.001017621453, 0.0005875240199, 0.0004154422186, 0.007055935247, -0.006382333571, 0.005628689138, -0.004757110573, 0.003684842005, -0.002127444524, -0.005826246777, 0.005138266683, -0.004342627949, 0.003363785145, -0.001942082259, 0.004595805436, -0.003884164518, 0.003008660899, -0.00173705118, -0.003363785145, 0.002605576769, -0.001504330449, 0.002127444524, -0.001228280668, -0.0008685255898},
    {0.009450674049, 0, -0.007041226712, 0.002489449578, 0, 0, 0.009471262597, 0, -0.005155502352, 0.001822745337, 0, 0, 0, 0.004509379509, 0, 0, -0.002928927908, 0, 0.001594306415, -0.0005636724387, 0, 0, 0, 0, 0.004644785208, 0, 0, 0, -0.003323537749, 0, 0, 0.002158701091, 0, -0.00117504804, 0.0004154422186, 0, 0, 0, 0, 0, 0.01276466714, 0, 0, 0, 0, -0.009710411295, 0, 0, 0, 0.006948204719, 0, 0, -0.004512991348, 0, 0.002456561337, -0.0008685255898},
    {0.009450674049, 0, 0, -0.007468348734, 0, 0, 0, 0, 0, 0.01093647202, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.005636724387, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.006231633279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.01823903739},
    {-0.01002344217, 0, 0.03747187483, -0.02896141836, 0, 0, 0.02841378779, 0, 0.005155502352, -0.006197334145, 0, 0, 0, 0.03156565657, 0, 0, -0.0113902752, 0, 0.002657177358, -0.0001878908129, 0, 0, 0, 0, 0.02322392604, 0, 0, 0, -0.001661768874, 0, 0, -0.006476103272, 0, 0.006462764219, -0.00290809553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02136290485, 0, 0, 0, -0.02779281887, 0, 0, 0.02437015328, 0, -0.01572199256, 0.006079679129},
    {0.01002344217, 0, -0.02875748534, 0.05360943398, 0, 0, -0.01894252519, 0, 0.01546650706, 0.01531106083, 0, 0, 0, -0.004509379509, 0, 0, 0.0370997535, 0, -0.02816608, 0.01089766715, 0, 0, 0, 0, 0, 0, 0, 0, 0.02991183974, 0, 0, -0.004317402181, 0, -0.01233800442, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03126692123, 0, 0, -0.04874030656, 0, 0.04127023046, -0.01823903739},
    {0.01002344217, 0, 0.04095763063, -0.04498262852, 0, 0, 0.01894252519, 0, -0.005155502352, -0.02114384591, 0, 0, 0, 0, 0, 0, 0.004556110079, 0, 0.0356061766, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03022181527, 0, 0.00411266814, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03790912732, 0, -0.05502697394, 0.03039839564},
    {-0.01002344217, 0, -0.01481446214, 0.04498262852, 0, 0, 0, 0, 0.02062200941, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, 0, 0.01062870943, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633407, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185872, -0.03039839564},
    {-0.01002344217, 0.03245159553, -0.01873593742, -0.02896141836, 0.021178387, -0.01640470803, 0.009471262597, 0.004464796006, -0.002577751176, -0.006197334145, 0.02087871931, -0.01764573846, 0.01366833024, -0.007891414141, -0.008489809874, 0.006576178451, -0.003796758399, 0.002301183095, -0.001328588679, -0.0001878908129, 0.01393435562, -0.01228894655, 0.01038605547, -0.00804500397, 0.004644785208, -0.001099156795, 0.0009289570415, -0.0007195670302, 0.0004154422186, -0.004827002382, 0.003738979967, -0.002158701091, 0.005596917993, -0.00323138211, -0.00290809553, 0, 0, 0, 0, 0, 0, 0.01281774291, -0.0113041867, 0.009553781488, -0.007400327319, 0.00427258097, -0.01838322174, 0.01553665807, -0.01203464359, 0.006948204719, 0.01816443978, -0.01407011456, 0.008123384426, -0.01361564495, 0.007860996278, 0.006079679129},
    {0.01002344217, -0.02490471285, 0.01437874267, 0.05360943398, -0.01411892467, 0.01093647202, -0.006314175065, 0.01339438802, -0.007733253528, 0.01531106083, -0.002982674187, 0.00252081978, -0.001952618605, 0.001127344877, 0.02765252359, -0.02141955267, 0.0123665845, -0.0243925408, 0.01408304, 0.01089766715, 0, 0, 0, 0, 0, 0.0197848223, -0.01672122675, 0.01295220654, -0.007477959935, -0.003218001588, 0.002492653312, -0.00143913406, -0.01068502526, 0.006169002209, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02068112446, -0.01747874033, 0.01353897404, -0.007816730308, -0.03632887957, 0.02814022911, -0.01624676885, 0.035741068, -0.02063511523, -0.01823903739},
    {0.01002344217, 0.0354703486, -0.02047881531, -0.04498262852, 0.01411892467, -0.01093647202, 0.006314175065, -0.004464796006, 0.002577751176, -0.02114384591, 0, 0, 0, 0, 0.00339592395, -0.00263047138, 0.00151870336, 0.03083585347, -0.0178030883, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02252601112, -0.01744857318, 0.01007393842, 0.003561675086, -0.00205633407, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02825579522, -0.02188684486, 0.01263637577, -0.04765475733, 0.02751348697, 0.03039839564},
    {-0.01002344217, -0.01282970056, 0.007407231071, 0.04498262852, 0, 0, 0, 0.01785918402, -0.0103110047, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, 0.009204732379, -0.005314354717, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.01780837543, -0.01028167035, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02978422333, -0.01719592936, -0.03039839564},
    {-0.01002344217, 0.03245159553, -0.03355039956, -0.008010605078, 0.021178387, 0.001093647202, -0.005682757558, -0.01696622482, 0.005671052587, 0.008384628549, 0.02087871931, -0.01092355238, 0.004990025324, -0.001878908129, -0.01625192176, 0.009582431457, -0.00466458889, 0.01089226665, -0.005580072453, -0.004697270322, 0.01393435562, -0.002457789311, -0.004154422186, 0.006436003176, -0.004644785208, -0.01209072474, 0.004644785208, -0.0007195670302, -0.0004154422186, 0.009654004764, -0.004985306623, 0.002158701091, -0.006614539446, 0.00323138211, 0.00290809553, 0, 0.01276466714, -0.0202632809, 0.02283413075, -0.02063511523, 0.01276466714, -0.001165249355, -0.007193573356, 0.01129083267, -0.01143686949, 0.007379912584, 0.001838322174, 0.003107331614, -0.004813857438, 0.003474102359, -0.002018271087, -0.0005211153539, 0.0009025982696, 0.001701955619, -0.0004913122674, -0.0008685255898},
    {0.01002344217, -0.02490471285, 0.0553363733, -0.004313402734, -0.01411892467, 0.01531106083, 0.008839845091, 0.008036632811, -0.01598205729, 0.0007290981347, -0.002982674187, 0.02772901758, -0.03015710957, 0.01916486291, -0.001455395978, -0.0116492304, 0.009763093026, 0.003528480745, 0.001328588679, -0.00263047138, 0, 0.01966231448, -0.007270238826, -0.007240503573, 0.009289570415, -0.002198313589, -0.01486331266, 0.01295220654, -0.006647075498, 0.004022501985, 0.006854796607, -0.005756536242, -0.004579296539, -0.00029376201, 0.00290809553, 0, 0, 0.0202632809, -0.03995972881, 0.04642900927, -0.03191166785, 0, -0.004110613346, -0.003039839564, 0.009082219892, -0.007768329036, 0.0004595805436, 0.00427258097, -0.002707794809, 0.0008685255898, -0.001009135544, -0.002345019093, 0.001805196539, 0.001276466714, 0.0002456561337, -0.0008685255898},
    {0.01002344217, 0.0354703486, -0.04923630065, -0.004313402734, 0.01411892467, -0.006561883212, -0.01641685517, -0.009822551213, 0.01495095682, 0.0007290981347, 0, 0.00336109304, 0.02820449096, -0.03532347282, -0.0004851319928, -0.01240079365, 0.008895262535, 0.00291483192, 0.002391459623, -0.00263047138, 0, 0, 0.02181071648, -0.004022501985, -0.009289570415, 0, -0.005573742249, -0.0143913406, 0.01412503543, 0.000804500397, 0.009347449919, -0.002878268121, -0.002544053633, -0.00381890613, 0.00290809553, 0, 0, 0, 0.02663981921, -0.05158778807, 0.04254889047, 0, 0, -0.009119518693, 0.008409462863, -0.003884164518, 0, 0.002330498711, 0.003008660899, -0.003474102359, -0.0003363785145, -0.002866134446, 0.001203464359, 0.0008509778095, 0.0009826245347, -0.0008685255898},
    {-0.01002344217, -0.01282970056, 0.0448791059, -0.008010605078, 0, 0.01749835523, 0.01325976764, -0.003571836805, -0.017528708, 0.008384628549, 0, 0, 0.008678304912, 0.02254689755, 0, -0.003006253006, -0.01920074962, 0.0006136488253, 0.01222301585, -0.004697270322, 0, 0, 0, 0.01609000794, 0.004644785208, 0, 0, -0.007195670302, -0.01080149768, 0, 0.002492653312, 0.01079350545, -0.0005088107266, -0.007344050249, 0.00290809553, 0, 0, 0, 0, 0.02579389404, -0.03191166785, 0, 0, 0, -0.01345514058, 0.01359457581, 0, 0, 0.006017321797, -0.003474102359, 0, -0.002084461416, -0.0009025982696, 0.0004254889047, 0.001719592936, -0.0008685255898},
    {-0.01002344217, -0.03245159553, -0.01873593742, -0.02896141836, 0.021178387, 0.01640470803, 0.009471262597, -0.004464796006, -0.002577751176, -0.006197334145, -0.02087871931, -0.01764573846, -0.01366833024, -0.007891414141, -0.008489809874, -0.006576178451, -0.003796758399, -0.002301183095, -0.001328588679, -0.0001878908129, 0.01393435562, 0.01228894655, 0.01038605547, 0.00804500397, 0.004644785208, 0.001099156795, 0.0009289570415, 0.0007195670302, 0.0004154422186, -0.004827002382, -0.003738979967, -0.002158701091, -0.005596917993, -0.00323138211, -0.00290809553, 0, 0, 0, 0, 0, 0, 0.01281774291, 0.0113041867, 0.009553781488, 0.007400327319, 0.00427258097, 0.01838322174, 0.01553665807, 0.01203464359, 0.006948204719, 0.01816443978, 0.01407011456, 0.008123384426, 0.01361564495, 0.007860996278, 0.006079679129},
    {0.01002344217, 0.02490471285, 0.01437874267, 0.05360943398, -0.01411892467, -0.01093647202, -0.006314175065, -0.01339438802, -0.007733253528, 0.01531106083, 0.002982674187, 0.00252081978, 0.001952618605, 0.001127344877, 0.02765252359, 0.02141955267, 0.0123665845, 0.0243925408, 0.01408304, 0.01089766715, 0, 0, 0, 0, 0, -0.0197848223, -0.01672122675, -0.01295220654, -0.007477959935, -0.003218001588, -0.002492653312, -0.00143913406, 0.01068502526, 0.006169002209, 0.008724286591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.02068112446, -0.01747874033, -0.01353897404, -0.007816730308, -0.03632887957, -0.02814022911, -0.01624676885, -0.035741068, -0.02063511523, -0.01823903739},
    {0.01002344217, -0.0354703486, -0.02047881531, -0.04498262852, 0.01411892467, 0.01093647202, 0.006314175065, 0.004464796006, 0.002577751176, -0.02114384591, 0, 0, 0, 0, 0.00339592395, 0.00263047138, 0.00151870336, -0.03083585347, -0.0178030883, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02252601112, 0.01744857318, 0.01007393842, -0.003561675086, -0.00205633407, -0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.02825579522, 0.02188684486, 0.01263637577, 0.04765475733, 0.02751348697, 0.03039839564},
    {-0.01002344217, 0.01282970056, 0.007407231071, 0.04498262852, 0, 0, 0, -0.01785918402, -0.0103110047, 0.02114384591, 0, 0, 0, 0, 0, 0, 0, -0.009204732379, -0.005314354717, 0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.01780837543, -0.01028167035, 0.01454047765, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.02978422333, -0.01719592936, -0.03039839564},
    {-0.01002344217, -0.03245159553, -0.03355039956, -0.008010605078, 0.021178387, -0.001093647202, -0.005682757558, 0.01696622482, 0.005671052587, 0.008384628549, -0.02087871931, -0.01092355238, -0.004990025324, -0.001878908129, -0.01625192176, -0.009582431457, -0.00466458889, -0.01089226665, -0.005580072453, -0.004697270322, 0.01393435562, 0.002457789311, -0.004154422186, -0.006436003176, -0.004644785208, 0.01209072474, 0.004644785208, 0.0007195670302, -0.0004154422186, 0.009654004764, 0.004985306623, 0.002158701091, 0.006614539446, 0.00323138211, 0.00290809553, 0, 0.01276466714, 0.0202632809, 0.02283413075, 0.02063511523, 0.01276466714, -0.001165249355, 0.007193573356, 0.01129083267, 0.01143686949, 0.007379912584, -0.001838322174, 0.003107331614, 0.004813857438, 0.003474102359, -0.002018271087, 0.0005211153539, 0.0009025982696, -0.001701955619, -0.0004913122674, -0.0008685255898},
    {0.01002344217, 0.02490471285, 0.0553363733, -0.004313402734, -0.01411892467, -0.01531106083, 0.008839845091, -0.008036632811, -0.01598205729, 0.0007290981347, 0.002982674187, 0.02772901758, 0.03015710957, 0.01916486291, -0.001455395978, 0.0116492304, 0.009763093026, -0.003528480745, 0.001328588679, -0.00263047138, 0, -0.01966231448, -0.007270238826, 0.007240503573, 0.009289570415, 0.002198313589, -0.01486331266, -0.01295220654, -0.006647075498, 0.004022501985, -0.006854796607, -0.005756536242, 0.004579296539, -0.00029376201, 0.00290809553, 0, 0, -0.0202632809, -0.03995972881, -0.04642900927, -0.03191166785, 0, 0.004110613346, -0.003039839564, -0.009082219892, -0.007768329036, -0.0004595805436, 0.00427258097, 0.002707794809, 0.0008685255898, -0.001009135544, 0.002345019093, 0.001805196539, -0.001276466714, 0.0002456561337, -0.0008685255898},
    {0.01002344217, -0.0354703486, -0.04923630065, -0.004313402734, 0.01411892467, 0.006561883212, -0.01641685517, 0.009822551213, 0.01495095682, 0.0007290981347, 0, 0.00336109304, -0.02820449096, -0.03532347282, -0.0004851319928, 0.01240079365, 0.008895262535, -0.00291483192, 0.002391459623, -0.00263047138, 0, 0, 0.02181071648, 0.004022501985, -0.009289570415, 0, -0.005573742249, 0.0143913406, 0.01412503543, 0.000804500397, -0.009347449919, -0.002878268121, 0.002544053633, -0.00381890613, 0.00290809553, 0, 0, 0, 0.02663981921, 0.05158778807, 0.04254889047, 0, 0, -0.009119518693, -0.008409462863, -0.003884164518, 0, 0.002330498711, -0.003008660899, -0.003474102359, -0.0003363785145, 0.002866134446, 0.001203464359, -0.0008509778095, 0.0009826245347, -0.0008685255898},
    {-0.01002344217, 0.01282970056, 0.0448791059, -0.008010605078, 0, -0.01749835523, 0.01325976764, 0.003571836805, -0.017528708, 0.008384628549, 0, 0, -0.008678304912, 0.02254689755, 0, 0.003006253006, -0.01920074962, -0.0006136488253, 0.01222301585, -0.004697270322, 0, 0, 0, -0.01609000794, 0.004644785208, 0, 0, 0.007195670302, -0.01080149768, 0, -0.002492653312, 0.01079350545, 0.0005088107266, -0.007344050249, 0.00290809553, 0, 0, 0, 0, -0.02579389404, -0.03191166785, 0, 0, 0, 0.01345514058, 0.01359457581, 0, 0, -0.006017321797, -0.003474102359, 0, 0.002084461416, -0.0009025982696, -0.0004254889047, 0.001719592936, -0.0008685255898},
    {-0.01002344217, -0.04528129609, -0.01132870634, -0.008010605078, -0.001411892467, 0.01640470803, 0.01452260265, 0.01339438802, 0.01185765541, 0.008384628549, -0.008948022562, -0.01428464542, -0.01453616073, -0.009394540645, -0.01237086582, -0.01258868446, -0.008135910855, -0.01027861782, -0.006642943396, -0.004697270322, -0.008360613374, 0.002457789311, 0.008308844372, 0.009654004764, 0.006502699291, 0.002198313589, 0.007431656332, 0.008634804363, 0.005816191061, 0.006436003176, 0.007477959935, 0.005036969212, 0.006105728719, 0.00411266814, 0.00290809553, 0.03527967624, 0.01914700071, 0.007880164793, 0.0009514221146, -0.002210905203, -0.002127444524, 0.01747874033, 0.007193573356, 0.0008685255898, -0.002018271087, -0.001942082259, 0.00643412761, 0.0007768329036, -0.001805196539, -0.00173705118, 0.000672757029, -0.001563346062, -0.001504330449, -0.001276466714, -0.001228280668, -0.0008685255898},
    {0.01002344217, 0.06037506145, -0.006100072647, -0.004313402734, -0.005647569867, -0.02187294404, 0.001262835013, -0.01785918402, 0.00103110047, 0.0007290981347, 0.04175743862, 0.0168054652, -0.0008678304912, -0.005260942761, 0.01455395978, -0.0007515632516, -0.004556110079, -0.0006136488253, -0.003720048302, -0.00263047138, 0.005573742249, -0.01966231448, -0.01038605547, 0.003218001588, 0.006502699291, -0.01758650871, -0.009289570415, 0.002878268121, 0.005816191061, -0.00804500397, 0.002492653312, 0.005036969212, 0.002035242906, 0.00411266814, 0.00290809553, -0.07055935247, -0.01276466714, 0.00450295131, 0.003805688459, -0.000736968401, -0.002127444524, -0.01165249355, 0.004110613346, 0.003474102359, -0.000672757029, -0.001942082259, 0.003676644349, 0.003107331614, -0.0006017321797, -0.00173705118, 0.002691028116, -0.0005211153539, -0.001504330449, -0.0004254889047, -0.001228280668, -0.0008685255898},
    {0.01002344217, -0.06037506145, -0.006100072647, -0.004313402734, -0.005647569867, 0.02187294404, 0.001262835013, 0.01785918402, 0.00103110047, 0.0007290981347, -0.04175743862, 0.0168054652, 0.0008678304912, -0.005260942761, 0.01455395978, 0.0007515632516, -0.004556110079, 0.0006136488253, -0.003720048302, -0.00263047138, 0.005573742249, 0.01966231448, -0.01038605547, -0.003218001588, 0.006502699291, 0.01758650871, -0.009289570415, -0.002878268121, 0.005816191061, -0.00804500397, -0.002492653312, 0.005036969212, -0.002035242906, 0.00411266814, 0.00290809553, 0.07055935247, -0.01276466714, -0.00450295131, 0.003805688459, 0.000736968401, -0.002127444524, -0.01165249355, -0.004110613346, 0.003474102359, 0.000672757029, -0.001942082259, -0.003676644349, 0.003107331614, 0.0006017321797, -0.00173705118, 0.002691028116, 0.0005211153539, -0.001504330449, 0.0004254889047, -0.001228280668, -0.0008685255898},
    {-0.01002344217, 0.04528129609, -0.01132870634, -0.008010605078, -0.001411892467, -0.01640470803, 0.01452260265, -0.01339438802, 0.01185765541, 0.008384628549, 0.008948022562, -0.01428464542, 0.01453616073, -0.009394540645, -0.01237086582, 0.01258868446, -0.008135910855, 0.01027861782, -0.006642943396, -0.004697270322, -0.008360613374, -0.002457789311, 0.008308844372, -0.009654004764, 0.006502699291, -0.002198313589, 0.007431656332, -0.008634804363, 0.005816191061, 0.006436003176, -0.007477959935, 0.005036969212, -0.006105728719, 0.00411266814, 0.00290809553, -0.03527967624, 0.01914700071, -0.007880164793, 0.0009514221146, 0.002210905203, -0.002127444524, 0.01747874033, -0.007193573356, 0.0008685255898, 0.002018271087, -0.001942082259, -0.00643412761, 0.0007768329036, 0.001805196539, -0.00173705118, 0.000672757029, 0.001563346062, -0.001504330449, 0.001276466714, -0.001228280668, -0.0008685255898},
    {0.07875561708, 0.06792194414, -0.004357194748, -0.003081001953, 0.07059462333, -0.03280941606, 0.01262835013, -0.02678877604, 0.03608851646, -0.007290981347, 0.02982674187, 0.01512491868, -0.03254364342, 0.0248015873, 0.01309856381, -0.02292267917, 0.01692269458, -0.02485277742, 0.01647449962, 0.01427970178, 0, 0.02457789311, -0.03635119413, 0.03620251786, -0.02322392604, 0.02198313589, -0.007431656332, 0, 0.001661768874, -0.03459351707, 0.01308642989, -0.003597835151, 0.03358150796, -0.0146881005, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0.02260837341, -0.03343823521, 0.03330147294, -0.02136290485, -0.004595805436, -0.01942082259, 0.02707794809, -0.01910756298, 0.009082219892, 0.007035057278, -0.008123384426, -0.01021173371, 0.001965249069, 0.006079679129},
    {-0.02147880466, 0.003773441341, 0.002178597374, -0.02156701367, 0, 0.09842824818, -0.08524136337, -0.03125357204, -0.01804425823, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.01164316783, 0.02367424242, -0.03351995272, -0.001380709857, -0.0007971532075, 0.003945707071, 0, 0, 0.03635119413, -0.06033752977, 0.04644785208, 0, 0.02415288308, 0, -0.01080149768, -0.007240503573, -0.03925928966, 0.02590441309, 0.01882599688, 0.01086919437, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0.03343823521, -0.0555024549, 0.0427258097, 0, -0.01553665807, 0, 0.006948204719, 0.003027406631, 0.01641513365, -0.01083117923, -0.006807822476, -0.003930498139, 0.006079679129},
    {0.07875561708, 0.03018753073, 0.06100072647, -0.003081001953, 0, 0.02187294404, 0.07577010078, 0.01785918402, -0.04124401881, -0.007290981347, 0, 0, 0.05206982947, -0.009018759019, 0, -0.01277657528, 0.02863840621, 0.001840946476, -0.02976038641, 0.01427970178, 0, 0, 0, 0.04022501985, -0.04644785208, 0, 0, 0.0143913406, 0.01827945762, 0, -0.01370959321, -0.03453921745, 0.004070485813, 0.03642648924, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016366, -0.0427258097, 0, 0, -0.03008660899, 0.02431871652, 0, 0.01407011456, 0, -0.003403911238, -0.009826245347, 0.006079679129},
    {-0.02147880466, 0.003773441341, -0.01960737637, 0.009243005859, 0, -0.01093647202, 0.009471262597, 0.1026903081, -0.06959928175, -0.06015059611, 0, -0.00504163956, 0.006508728684, -0.004509379509, 0.04657267131, 0.02141955267, -0.02896384264, -0.007824022522, -0.02630605585, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0.0334424535, -0.04317402181, 0.02991183974, 0.02896201429, -0.002492653312, -0.004317402181, -0.06054847647, 0.01850700663, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03495748066, -0.04512991348, 0.03126692123, -0.01210962652, -0.02814022911, 0.02707794809, 0.02382737866, 0, -0.01823903739},
    {-0.02147880466, -0.01509376536, 0.01307158424, 0.009243005859, 0, -0.01093647202, 0.009471262597, -0.008929592012, 0.1237320564, -0.06015059611, 0, 0, -0.006508728684, 0.006764069264, 0, 0.05749458874, 0.01269202093, -0.0266937239, 0.00637722566, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0, 0.04317402181, -0.04486775961, 0, 0.01994122649, 0.02158701091, -0.01424670034, -0.06169002209, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.04512991348, -0.04690038185, 0, -0.03752030548, 0.01624676885, 0.01191368933, 0.02063511523, -0.01823903739},
    {0.07875561708, 0.03018753073, 0.01742877899, 0.05853903711, 0, 0.02187294404, -0.01894252519, 0.01785918402, 0.0103110047, 0.08384628549, 0, 0, 0, 0, 0, 0.005260942761, -0.004556110079, 0.0533874478, 0.03082325736, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03489714636, -0.03022181527, 0.02849340069, 0.01645067256, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.04377368973, -0.03790912732, -0.02382737866, -0.01375674349, 0.03039839564},
    {0.07875561708, -0.06792194414, -0.004357194748, -0.003081001953, 0.07059462333, 0.03280941606, 0.01262835013, 0.02678877604, 0.03608851646, -0.007290981347, -0.02982674187, 0.01512491868, 0.03254364342, 0.0248015873, 0.01309856381, 0.02292267917, 0.01692269458, 0.02485277742, 0.01647449962, 0.01427970178, 0, -0.02457789311, -0.03635119413, -0.03620251786, -0.02322392604, -0.02198313589, -0.007431656332, 0, 0.001661768874, -0.03459351707, -0.01308642989, -0.003597835151, -0.03358150796, -0.0146881005, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, -0.02260837341, -0.03343823521, -0.03330147294, -0.02136290485, 0.004595805436, -0.01942082259, -0.02707794809, -0.01910756298, 0.009082219892, -0.007035057278, -0.008123384426, 0.01021173371, 0.001965249069, 0.006079679129},
    {-0.02147880466, -0.003773441341, 0.002178597374, -0.02156701367, 0, -0.09842824818, -0.08524136337, 0.03125357204, -0.01804425823, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.01164316783, -0.02367424242, -0.03351995272, 0.001380709857, -0.0007971532075, 0.003945707071, 0, 0, 0.03635119413, 0.06033752977, 0.04644785208, 0, 0.02415288308, 0, -0.01080149768, -0.007240503573, 0.03925928966, 0.02590441309, -0.01882599688, 0.01086919437, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0.03343823521, 0.0555024549, 0.0427258097, 0, -0.01553665807, 0, 0.006948204719, 0.003027406631, -0.01641513365, -0.01083117923, 0.006807822476, -0.003930498139, 0.006079679129},
    {0.07875561708, -0.03018753073, 0.06100072647, -0.003081001953, 0, -0.02187294404, 0.07577010078, -0.01785918402, -0.04124401881, -0.007290981347, 0, 0, -0.05206982947, -0.009018759019, 0, 0.01277657528, 0.02863840621, -0.001840946476, -0.02976038641, 0.01427970178, 0, 0, 0, -0.04022501985, -0.04644785208, 0, 0, -0.0143913406, 0.01827945762, 0, 0.01370959321, -0.03453921745, -0.004070485813, 0.03642648924, -0.0178640154, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016366, -0.0427258097, 0, 0, 0.03008660899, 0.02431871652, 0, -0.01407011456, 0, 0.003403911238, -0.009826245347, 0.006079679129},
    {-0.02147880466, -0.003773441341, -0.01960737637, 0.009243005859, 0, 0.01093647202, 0.009471262597, -0.1026903081, -0.06959928175, -0.06015059611, 0, -0.00504163956, -0.006508728684, -0.004509379509, 0.04657267131, -0.02141955267, -0.02896384264, 0.007824022522, -0.02630605585, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0.0334424535, 0.04317402181, 0.02991183974, 0.02896201429, 0.002492653312, -0.004317402181, 0.06054847647, 0.01850700663, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03495748066, 0.04512991348, 0.03126692123, -0.01210962652, 0.02814022911, 0.02707794809, -0.02382737866, 0, -0.01823903739},
    {-0.02147880466, 0.01509376536, 0.01307158424, 0.009243005859, 0, 0.01093647202, 0.009471262597, 0.008929592012, 0.1237320564, -0.06015059611, 0, 0, 0.006508728684, 0.006764069264, 0, -0.05749458874, 0.01269202093, 0.0266937239, 0.00637722566, 0.0005636724387, 0, 0, 0, 0, 0, 0, 0, -0.04317402181, -0.04486775961, 0, -0.01994122649, 0.02158701091, 0.01424670034, -0.06169002209, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04512991348, -0.04690038185, 0, 0.03752030548, 0.01624676885, -0.01191368933, 0.02063511523, -0.01823903739},
    {0.07875561708, -0.03018753073, 0.01742877899, 0.05853903711, 0, -0.02187294404, -0.01894252519, -0.01785918402, 0.0103110047, 0.08384628549, 0, 0, 0, 0, 0, -0.005260942761, -0.004556110079, -0.0533874478, 0.03082325736, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.03489714636, -0.03022181527, -0.02849340069, 0.01645067256, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04377368973, -0.03790912732, 0.02382737866, -0.01375674349, 0.03039839564},
    {0.07875561708, -0.03773441341, -0.05664353172, -0.003081001953, 0.042356774, 0.0546823601, 0.03788505039, 0.04464796006, 0.005155502352, -0.007290981347, 0.04175743862, -0.00504163956, -0.024733169, -0.02029220779, 0.02959305156, 0.0101461039, 0.002169576228, 0.0266937239, 0.01328588679, 0.01427970178, -0.05573742249, -0.02457789311, -0.005193027733, 0.004022501985, 0.004644785208, -0.002198313589, -0.01300539858, -0.01583047467, -0.01080149768, -0.01689450834, -0.0267960231, -0.01942830982, -0.02951102214, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, -0.05127097164, -0.02260837341, -0.004776890744, 0.00370016366, 0.00427258097, -0.03676644349, -0.007768329036, 0.006017321797, 0.006948204719, -0.009082219892, 0.007035057278, 0.008123384426, 0.006807822476, 0.007860996278, 0.006079679129},
    {-0.02147880466, 0, -0.004357194748, -0.02156701367, -0.127070322, 0, 0.02841378779, 0, 0.03608851646, 0.03098667072, 0, 0.04537475604, 0, -0.02029220779, -0.04220648338, 0, -0.00618329225, 0, 0.001594306415, 0.003945707071, 0.08360613374, 0, -0.01038605547, 0, 0.004644785208, 0, 0.02415288308, 0, -0.01080149768, 0.04344302144, 0, -0.01942830982, 0, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, 0.07690645746, 0, -0.009553781488, 0, 0.00427258097, 0, -0.01553665807, 0, 0.006948204719, -0.01816443978, 0, 0.008123384426, 0, 0.007860996278, 0.006079679129},
    {0.07875561708, 0.03773441341, -0.05664353172, -0.003081001953, 0.042356774, -0.0546823601, 0.03788505039, -0.04464796006, 0.005155502352, -0.007290981347, -0.04175743862, -0.00504163956, 0.024733169, -0.02029220779, 0.02959305156, -0.0101461039, 0.002169576228, -0.0266937239, 0.01328588679, 0.01427970178, -0.05573742249, 0.02457789311, -0.005193027733, -0.004022501985, 0.004644785208, 0.002198313589, -0.01300539858, 0.01583047467, -0.01080149768, -0.01689450834, 0.0267960231, -0.01942830982, 0.02951102214, -0.02173838874, -0.0178640154, 0, 0, 0, 0, 0, 0, -0.05127097164, 0.02260837341, -0.004776890744, -0.00370016366, 0.00427258097, 0.03676644349, -0.007768329036, -0.006017321797, 0.006948204719, -0.009082219892, -0.007035057278, 0.008123384426, -0.006807822476, 0.007860996278, 0.006079679129},
    {-0.02147880466, -0.0188672067, 0.006535792122, 0.009243005859, 0.01411892467, 0, -0.003157087532, -0.1116199001, -0.05413277469, -0.06015059611, -0.008948022562, -0.00252081978, 0.0006508728684, 0.001127344877, -0.02765252359, 0.03607503608, 0.03742518993, -0.01886970138, 0.01992883019, 0.0005636724387, 0, 0, 0, 0, 0, 0.05935446691, 0.01672122675, -0.004317402181, -0.007477959935, 0.003218001588, 0.0224338798, 0.01870874279, 0.04630177612, 0.04318301547, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.06204337339, 0.01747874033, -0.004512991348, -0.007816730308, 0.03632887957, -0.00938007637, -0.01624676885, -0.01191368933, -0.02063511523, -0.01823903739},
    {-0.02147880466, 0.0188672067, 0.006535792122, 0.009243005859, 0.01411892467, 0, -0.003157087532, 0.1116199001, -0.05413277469, -0.06015059611, 0.008948022562, -0.00252081978, -0.0006508728684, 0.001127344877, -0.02765252359, -0.03607503608, 0.03742518993, 0.01886970138, 0.01992883019, 0.0005636724387, 0, 0, 0, 0, 0, -0.05935446691, 0.01672122675, 0.004317402181, -0.007477959935, 0.003218001588, -0.0224338798, 0.01870874279, -0.04630177612, 0.04318301547, 0.04611408627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.06204337339, 0.01747874033, 0.004512991348, -0.007816730308, 0.03632887957, 0.00938007637, -0.01624676885, 0.01191368933, -0.02063511523, -0.01823903739},
    {0.07875561708, 0, -0.03485755798, 0.05853903711, -0.02823784933, 0, 0.006314175065, 0, -0.02062200941, 0.08384628549, 0, 0, 0, 0, -0.006791847899, 0, 0.00151870336, 0, -0.06164651471, -0.03194143819, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.04505202223, 0, 0.01007393842, 0, -0.03290134512, -0.06439354388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.05651159044, 0, 0.01263637577, 0, 0.02751348697, 0.03039839564},
    {0.07875561708, -0.03773441341, -0.02178597374, -0.0523770332, 0.042356774, 0.0546823601, 0, 0.04464796006, 0.02577751176, 0.02916392539, 0.04175743862, 0.02856929084, 0.02733666047, 0.01878908129, -0.009217507864, -0.007891414141, -0.003471321965, -0.02485277742, -0.01434875774, -0.01503126503, -0.05573742249, -0.004915578621, -0.01973350538, -0.03942051945, -0.03251349645, -0.02418144948, -0.009289570415, -0.01583047467, -0.01412503543, -0.002413501191, -0.0006231633279, -0.002158701091, 0.007123350172, 0.00411266814, 0.005400748842, 0, -0.05105866857, -0.04052656179, -0.01141706538, 0.01031755761, 0.01276466714, 0.004660997422, -0.01438714671, -0.005645416334, 0.005718434747, 0.007379912584, 0.003676644349, -0.001553665807, 0.002406928719, 0.003474102359, 0.001009135544, 0.0002605576769, 0.0009025982696, -0.0008509778095, -0.0004913122674, -0.0008685255898},
    {-0.02147880466, 0, -0.02178597374, 0.003081001953, -0.127070322, 0, 0.04735631299, 0, 0.02577751176, 0.01275921736, 0, -0.03529147692, 0, 0.002254689755, 0.05093885925, 0, -0.009437656592, 0, -0.01434875774, -0.01296446609, 0.08360613374, 0, 0.04777585514, 0, -0.03251349645, 0, 0.009289570415, 0, -0.01412503543, -0.01448100715, 0, -0.002158701091, 0, 0.00411266814, 0.005400748842, 0, 0.07658800285, 0, -0.02283413075, 0, 0.01276466714, -0.006991496133, 0, -0.01129083267, 0, 0.007379912584, 0, -0.003107331614, 0, 0.003474102359, 0.002018271087, 0, 0.0009025982696, 0, -0.0004913122674, -0.0008685255898},
    {0.07875561708, 0.03773441341, -0.02178597374, -0.0523770332, 0.042356774, -0.0546823601, 0, -0.04464796006, 0.02577751176, 0.02916392539, -0.04175743862, 0.02856929084, -0.02733666047, 0.01878908129, -0.009217507864, 0.007891414141, -0.003471321965, 0.02485277742, -0.01434875774, -0.01503126503, -0.05573742249, 0.004915578621, -0.01973350538, 0.03942051945, -0.03251349645, 0.02418144948, -0.009289570415, 0.01583047467, -0.01412503543, -0.002413501191, 0.0006231633279, -0.002158701091, -0.007123350172, 0.00411266814, 0.005400748842, 0, -0.05105866857, 0.04052656179, -0.01141706538, -0.01031755761, 0.01276466714, 0.004660997422, 0.01438714671, -0.005645416334, -0.005718434747, 0.007379912584, -0.003676644349, -0.001553665807, -0.002406928719, 0.003474102359, 0.001009135544, -0.0002605576769, 0.0009025982696, 0.0008509778095, -0.0004913122674, -0.0008685255898},
    {-0.02147880466, -0.0188672067, 0.01089298687, 0.003081001953, 0.01411892467, -0.1093647202, -0.07892718831, 0.02232398003, -0.01288875588, 0.01275921736, -0.008948022562, -0.02772901758, -0.005857855816, 0.01916486291, 0.001455395978, 0.03832972583, 0.03482169846, -0.01242638871, 0.007174378868, -0.01296446609, 0, 0.05898694345, 0.007270238826, 0.05068352501, 0.06502699291, -0.006594940767, 0.01486331266, -0.004317402181, -0.001661768874, -0.004022501985, -0.008101123263, -0.01151307248, 0.003561675086, -0.00205633407, 0.005400748842, 0, 0, 0.06078984269, 0.03995972881, -0.01547633642, -0.03191166785, 0, -0.01233184004, 0.003039839564, -0.003027406631, -0.007768329036, 0.001378741631, -0.00427258097, 0.0009025982696, 0.0008685255898, 0.001009135544, 0.0007816730308, 0.001805196539, -0.0004254889047, 0.0002456561337, -0.0008685255898},
    {-0.02147880466, 0.0188672067, 0.01089298687, 0.003081001953, 0.01411892467, 0.1093647202, -0.07892718831, -0.02232398003, -0.01288875588, 0.01275921736, 0.008948022562, -0.02772901758, 0.005857855816, 0.01916486291, 0.001455395978, -0.03832972583, 0.03482169846, 0.01242638871, 0.007174378868, -0.01296446609, 0, -0.05898694345, 0.007270238826, -0.05068352501, 0.06502699291, 0.006594940767, 0.01486331266, 0.004317402181, -0.001661768874, -0.004022501985, 0.008101123263, -0.01151307248, -0.003561675086, -0.00205633407, 0.005400748842, 0, 0, -0.06078984269, 0.03995972881, 0.01547633642, -0.03191166785, 0, 0.01233184004, 0.003039839564, 0.003027406631, -0.007768329036, -0.001378741631, -0.00427258097, -0.0009025982696, 0.0008685255898, 0.001009135544, -0.0007816730308, 0.001805196539, 0.0004254889047, 0.0002456561337, -0.0008685255898},
    {0.07875561708, 0, 0.04357194748, -0.0523770332, -0.02823784933, 0, 0.06314175065, 0, -0.05155502352, 0.02916392539, 0, -0.00672218608, 0, -0.06012506013, 0.0009702639856, 0, -0.01258354212, 0, 0.02869751547, -0.01503126503, 0, 0, -0.04362143295, 0, -0.06502699291, 0, 0.0111474845, 0, 0.03157360861, -0.001609000794, 0, -0.002878268121, 0, -0.008225336279, 0.005400748842, 0, 0, 0, -0.05327963842, 0, 0.04254889047, 0, 0, 0.01823903739, 0, -0.003884164518, 0, -0.004660997422, 0, -0.003474102359, 0.000672757029, 0, 0.001203464359, 0, 0.0009826245347, -0.0008685255898},
    {0.1073940233, -0.188672067, -0.1089298687, -0.07702504883, -0.07059462333, -0.0546823601, -0.03157087532, -0.04464796006, -0.02577751176, -0.01822745337, 0.08948022562, -0.01512491868, 0.05857855816, 0.0744047619, -0.01309856381, -0.02818362193, 0.009763093026, 0.06903549284, 0.03985766038, 0.0597492785, 0, 0.07373367932, 0.03635119413, -0.01206750595, -0.02322392604, 0.06594940767, 0.007431656332, 0.04317402181, 0.04652952848, 0.03459351707, 0.0155790832, 0.02518484606, -0.02544053633, -0.0146881005, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, 0.06782512022, 0.03343823521, -0.01110049098, -0.02136290485, -0.01378741631, 0.01942082259, -0.009025982696, -0.01910756298, -0.009082219892, -0.002345019093, -0.008123384426, 0.003403911238, 0.001965249069, 0.006079679129},
    {0.1073940233, 0.188672067, -0.1089298687, -0.07702504883, -0.07059462333, 0.0546823601, -0.03157087532, 0.04464796006, -0.02577751176, -0.01822745337, -0.08948022562, -0.01512491868, -0.05857855816, 0.0744047619, -0.01309856381, 0.02818362193, 0.009763093026, -0.06903549284, 0.03985766038, 0.0597492785, 0, -0.07373367932, 0.03635119413, 0.01206750595, -0.02322392604, -0.06594940767, 0.007431656332, -0.04317402181, 0.04652952848, 0.03459351707, -0.0155790832, 0.02518484606, 0.02544053633, -0.0146881005, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, -0.06782512022, 0.03343823521, 0.01110049098, -0.02136290485, 0.01378741631, 0.01942082259, 0.009025982696, -0.01910756298, -0.009082219892, 0.002345019093, -0.008123384426, -0.003403911238, 0.001965249069, 0.006079679129},
    {0.1073940233, 0, 0.2178597374, -0.07702504883, 0, 0, -0.09471262597, 0, 0.05155502352, -0.01822745337, 0, -0.09074951207, 0, -0.0946969697, 0.02328633566, 0, -0.02278055039, 0, -0.07971532075, 0.0597492785, 0, 0, -0.07270238826, 0, 0.04644785208, 0, -0.04830576616, 0, -0.0781031371, 0.01448100715, 0, 0.04317402181, 0, 0.029376201, -0.03281993527, 0, 0, 0, 0, 0, 0, 0, 0, -0.06687647042, 0, 0.0427258097, 0, 0.03107331614, 0, 0.006948204719, -0.006054813261, 0, -0.01083117923, 0, -0.003930498139, 0.006079679129},
    {0.1073940233, 0, 0, 0.2310751465, 0, 0, 0, 0, 0, -0.1093647202, 0, 0.01008327912, 0, -0.004509379509, -0.09314534262, 0, -0.1041396589, 0, 0, -0.09244227994, 0, 0, 0, 0, 0, 0, -0.06688490699, 0, 0.02991183974, -0.05792402858, 0, -0.06476103272, 0, 0, 0.08350388594, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.06991496133, 0, 0.03126692123, 0.02421925305, 0, 0.02707794809, 0, 0, -0.01823903739}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.61248608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.291287847, 0, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.749545417, 0, -1.673320053, -1.183215957, 7.745966692, 0, 0.3464101615, 0, 0.2828427125, 0.2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.449489743, 0, 0, 0, 7.09929574, 0, -0.4140393356, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0, 4.38178046, -0.7745966692, 0, 0, 4.76235236, 0, -0.7406560798, 0.1309307341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.121320344, 0, 0, 0, 0, 0, 7.171371656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.558845727, 0, 1.58113883, 2.459674775, 0, 0, 0, 0, 5.345224838, -1.209486314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.272792206, 0, 0, 3.834057903, 0, 0, 0, 0, 0, 5.184592559, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 5.802884575, 0, 0, 0, -2.102294665, 0, -1.716516406, 0, 0, 9.810708435, 0, 0.3058388709, 0, 0, 0.2648642317, 0, 0.2162607396, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.265686062, 0, 1.772810521, -1.253566341, 2.393567769, 0, -2.783133725, 0, -0.5244044241, 0.3708099244, 0, 9.401747558, 0, 0.8409178659, -0.3015611935, 0, 0.3506418906, 0, 0.06606874726, -0.04671765922, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.297564437, 0, 0, 0, 5.014265364, 0, -0.3838247368, 0, 0, 0, 0, 8.206518066, 0, 0, -0.6317380553, 0, 0.04835737146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.483239697, 0, 4.062019202, -0.8206518066, 0, 0, 5.886425061, 0, -1.201561484, 0.2427520781, 0, 0, 0, 5.138093031, 0, 0, -0.7416198487, 0, 0.1513825177, -0.03058388709, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122, 0, -0.7928249672, 2.242447642, 2.140872096, 0, 0.09574271078, 0, -1.876166304, -2.155806114, 0, 0, 0, 0, 9.440358593, 0, 0.4221856709, 0, 0.5909368403, 0.5223193088, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.160576915, 0, 0, 0, 1.962141687, 0, 2.746426249, 0, 0, 0, 0, 0, 0, 0, 8.652231568, 0, -0.8650429583, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.326649916, 0, 2.0761031, 1.46802658, 0, 0, 1.316244658, 0, 4.912956631, -1.411306386, 0, 0, 0, 0, 0, 0, 5.804093383, 0, -1.547435887, 0.3419382528, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.005089091, 0, 0, 0, 0, 0, 4.45964044, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.427928244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.148912529, 0, 0.7491491773, 2.754588009, 0, 0, 0, 0, 3.324019727, 2.068384463, 0, 0, 0, 0, 0, 0, 0, 0, 6.281806821, -1.776763281, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083152, 0, 0, 3.11415465, 0, 0, 0, 0, 0, 5.52707621, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.802884575, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.415933504, 0, -2.205435462, -1.559478371, 8.337498959, 0, 1.065326539, 0, 0.8698354768, 0.6150665641, 0, -2.183269719, 0, -0.2789679717, -1.89076704, 0, -0.2415933504, 0, -0.1972601445, -0.1394839859, 11.84905059, 0, 0.2453266907, 0, 0.0313466947, 0, 0.2194268629, 0, 0.02803733608, 0.1900292375, 0, 0.0242810453, 0, 0.01982539047, 0.01401866804, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.455253321, 0, 0, 0, 5.285312083, 0, -1.362770288, 0, 0, 2.336664289, 0, -3.933529339, 0, 0, -1.198596118, 0, 0.3090472522, 0, 0, 0, 11.55277744, 0, 0.8103287315, 0, -0.2348435972, 0, 0.3953345733, 0, 0, 0.1204634423, 0, -0.03106041751, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.927248223, 0, 2.99085909, -1.244033379, 0.9501461876, 0, -0.1699673171, 0, -1.179610733, 0.4906533815, 0, 4.976133515, 0, -3.226822114, -0.2154729018, 0, 0.03854496447, 0, 0.2675105694, -0.1112697281, 0, 0, 10.76366487, 0, 1.375330648, 0, -0.5001202357, 0, 0.3243078247, 0.02165584145, 0, -0.003873914688, 0, -0.02688582383, 0.01118302844, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.7726181305, 0, 0, 0, 3.358890123, 0, -0.3047247001, 0, 0, 0, 0, 7.329699161, 0, 0, -0.7617246811, 0, 0.06910506641, 0, 0, 0, 0, 0, 9.059750639, 0, 0, 0, -0.7366624832, 0, 0, 0.07655621093, 0, -0.006945320497, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.261680124, 0, 3.685603542, -0.8144110179, 0, 0, 6.008565315, 0, -1.45362164, 0.3212080372, 0, 0, 0, 6.992941047, 0, 0, -1.362614534, 0, 0.3296504023, -0.07284313591, 0, 0, 0, 0, 5.402191543, 0, 0, 0, -0.7028170192, 0, 0, 0.1369479134, 0, -0.03313111202, 0.007321010617, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 3.154200309, 0, 0, 0, -0.7618117503, 0, 5.598150205, 0, 0, 2.133072901, 0, 0.06649638116, 0, 0, -2.361089769, 0, -3.338422985, 0, 0, 0, 0, 0, 0, 0, 11.57662221, 0, 0.3608894393, 0, 0, 0.5556256398, 0, 0.5954371961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.759328876, 0, 0.9636241117, 1.362770288, 0.8673608331, 0, -1.008528448, 0, 1.710263138, -2.015564437, 0, 2.044155432, 0, 0.1828348201, 2.688218958, 0, -3.12574097, 0, -1.019905069, 0.9040166103, 0, 0, 0, 0, 0, 0, 11.09404895, 0, 0.9922819038, -0.6326076195, 0, 0.7355678929, 0, 0.1819090682, -0.1470047265, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.7053006306, 0, 0, 0, 1.81702705, 0, 1.251784441, 0, 0, 0, 0, 1.784285142, 0, 0, 5.631527706, 0, -0.7464940732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.683679823, 0, 0, -1.325244481, 0, 0.1331438047, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.151751107, 0, 2.207940217, 0.8921425712, 0, 0, 2.133072901, 0, 3.918705143, -1.319496657, 0, 0, 0, 1.117139204, 0, 0, 6.611051354, 0, -2.33689609, 0.5918177923, 0, 0, 0, 0, 0, 0, 0, 0, 6.06294258, 0, 0, -1.555751791, 0, 0.4168060373, -0.09623718381, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.573591585, 0, -0.4309458037, 3.047247001, 0.7757911135, 0, 0.03469443332, 0, -1.529705854, 0.9013878189, 0, 0, 0, 0, 4.515610783, 0, 0.2019442534, 0, -1.837309973, -2.748253414, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.89206303, 0, 0.487107867, 0, 0.8948742928, 0.903959544, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.6308400619, 0, 0, 0, 0.7110243003, 0, 2.239260082, 0, 0, 0, 0, 0, 0, 0, 4.138625644, 0, 2.689546405, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.982740661, 0, -1.309961832, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.030157507, 0, 1.128481009, 1.994891435, 0, 0, 0.4769696007, 0, 4.00571021, 0.5900968444, 0, 0, 0, 0, 0, 0, 2.776274482, 0, 4.811206872, -1.799154185, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.696626016, 0, -2.343330963, 0.5917804336, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0.5463235193, 0, 0, 0, 0, 0, 2.693411273, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.718890898, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.453835201, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8921425712, 0, 0.407205509, 2.533852435, 0, 0, 0, 0, 2.007550233, 3.293361529, 0, 0, 0, 0, 0, 0, 0, 0, 5.007965594, 1.416466573, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.046472719, -2.277763951, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7284313591, 0, 0, 2.53908227, 0, 0, 0, 0, 0, 5.007137762, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.939240681, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.276779364, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.738612788, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.479019946, 2.80624304, -0.5400617249, -0.3818813079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.145643924, 3.622844187, 2.091650066, -0.2958039892, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.322875656, 0, 4.830458915, 0.3415650255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9354143467, 0, 0, 2.173706512, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.620185175, 0, 0, 3.764970119, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.374772708, 2.898275349, -0.8366600265, -0.5916079783, 3.872983346, -0.6, 0.1732050808, -0.4898979486, 0.1414213562, 0.1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.161895004, 1.224744871, 1.414213562, -0.5, 4.582575695, 3.54964787, -1.024695077, -0.2070196678, -0.2390457219, 0.08451542547, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9, -2.846049894, 2.19089023, -0.3872983346, 0, 5.499090834, 2.38117618, 0.4810702354, -0.3703280399, 0.06546536707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.598076211, 0, -1.58113883, -1.118033989, 0, 0, 6.873863542, 0, 0.2672612419, 0.1889822365, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623059, 1.060660172, -0.2041241452, 1.58771324, 0, 0, 0, 3.585685828, -0.6900655593, -0.7807200584, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7794228634, 1.369306394, 0.790569415, 1.229837388, 0, 0, 0, 4.629100499, 2.672612419, -0.6047431568, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.9, 0, 1.825741858, -1.420093894, 0, 0, 0, 0, 6.172133998, 0.6982972488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6363961031, 0, 0, 1.917028951, 0, 0, 0, 0, 0, 2.592296279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.102270384, 0, 0, 3.320391543, 0, 0, 0, 0, 0, 4.489988864, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.284523258, 2.901442287, -1.005089091, -0.7107053122, 4.071076113, -1.051147332, 0.3641281172, -0.8582582029, 0.2973093627, 0.2102294665, 4.905354218, -0.5922544268, 0.1529194355, -0.05297284634, -0.5129073792, 0.1324321158, -0.04587583064, 0.1081303698, -0.03745745886, -0.02648642317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.132843031, 1.535298947, 0.8864052604, -0.6267831705, 1.196783885, 3.151884357, -1.391566863, -0.4541475531, -0.262202212, 0.1854049622, 5.562148865, 4.700873779, -1.213760391, 0.4204589329, -0.1507805968, -0.3971001033, 0.1753209453, 0.05721721353, 0.03303437363, -0.02335882961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9574271078, 0.6487822187, 1.872872943, -0.5297284634, -4.045867969, 2.507132682, 0.9046835806, -0.1919123684, -0.5540032878, 0.1566957926, 0, 6.35674156, 4.103259033, -1.421410624, 0.5097314515, -0.3158690277, -0.1139794176, 0.02417868573, 0.06979785357, -0.01974181423, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7416198487, 3.51781182, 2.031009601, -0.4103259033, 0, -3.398529094, 2.943212531, -1.040582769, -0.600780742, 0.1213760391, 0, 0, 7.416198487, 2.569046516, 0, 0.4281744193, -0.3708099244, 0.131101106, 0.07569125885, -0.01529194355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.712697677, 0, 4.69041576, 0.947607083, 0, 0, -1.699264547, 0, -1.387443693, -0.2803059553, 0, 0, 0, 8.899438185, 0, 0, 0.2140872096, 0, 0.1748014747, 0.03531523089, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561, 1.373213125, -0.3964124836, 1.121223821, 1.070436048, -0.1658312395, 0.04787135539, 3.249615362, -0.938083152, -1.077903057, 0, 0, 0, 0, 4.720179297, -0.7312470323, 0.2110928355, -1.023532631, 0.2954684201, 0.2611596544, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8563488386, 0.5802884575, 0.6700593943, 0.947607083, 1.266557013, 0.9810708435, -0.2832107578, 1.373213125, 1.585649934, -0.9109943547, 0, 0, 0, 0, 5.584991462, 4.326115784, -1.248842056, -0.4325214792, -0.4994327848, 0.2207201931, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6633249581, -1.348468519, 1.03805155, 0.7340132902, 0, 1.519868415, 0.6581223291, -3.191058938, 2.456478315, -0.7056531928, 0, 0, 0, 0, 0, 6.701989754, 2.902046692, 1.005089091, -0.7737179433, 0.1709691264, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.914854216, 0, -0.7491491773, 2.118913853, 0, 0, 1.899835519, 0, -1.772810521, -2.037045304, 0, 0, 0, 0, 0, 0, 8.377487193, 0, 0.5583828286, 0.4935453557, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7416198487, 0.5025445457, -0.09671474291, 1.778078914, 0, 0, 0, 2.22982022, -0.4291291015, 1.33513643, 0, 0, 0, 0, 0, 0, 0, 4.213964122, -0.8109777734, -1.146895766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562647, 0.6487822187, 0.3745745886, 1.377294005, 0, 0, 0, 2.878685526, 1.662009863, 1.034192231, 0, 0, 0, 0, 0, 0, 0, 5.440204289, 3.140903411, -0.8883816403, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.6633249581, 0, 0.8650429583, -1.590362129, 0, 0, 0, 0, 3.838247368, -1.194182326, 0, 0, 0, 0, 0, 0, 0, 0, 7.253605718, 1.025814758, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041576, 0, 0, 1.557077325, 0, 0, 0, 0, 0, 2.763538105, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2.901442287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8124038405, 0, 0, 2.696937038, 0, 0, 0, 0, 0, 4.786588406, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.025445457, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.207966752, 2.864944705, -1.102717731, -0.7797391854, 4.168749479, -1.383899769, 0.5326632696, -1.12994943, 0.4349177384, 0.3075332821, 5.166559139, -1.09163486, 0.3623900255, -0.1394839859, -0.9453835201, 0.3138389682, -0.1207966752, 0.2562484445, -0.09863007227, -0.06974199293, 5.924525297, -0.5805489101, 0.1226633454, -0.0407205509, 0.01567334735, -0.5192587309, 0.1097134314, -0.03642156795, 0.01401866804, 0.09501461876, -0.03154200309, 0.01214052265, -0.02575393768, 0.009912695236, 0.007009334021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.092647039, 1.72762666, 0.4987228587, -0.7053006306, 1.616046764, 2.642656041, -1.445436167, -0.6813851439, -0.1966989481, 0.2781743201, 1.168332145, 4.513924036, -1.96676467, 0.8200920804, -0.3664849581, -0.5992980588, 0.3277941116, 0.1545236261, 0.04460712856, -0.06308400619, 6.549809158, 5.776388722, -1.220484876, 0.4051643657, -0.1559478371, -0.1174217986, -0.4536664352, 0.1976672867, -0.08242235528, 0.03683312416, 0.06023172117, -0.03294454778, -0.01553020876, -0.004483185103, 0.006340181176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9636241117, 0.9141741003, 1.495429545, -0.6220166894, 0.4750730938, 2.870322282, -0.08498365856, -0.3605551275, -0.5898053665, 0.2453266907, -5.151860505, 2.488066758, 2.890872335, -1.613411057, -0.1077364509, -0.6509279091, 0.01927248223, 0.08176621726, 0.1337552847, -0.05563486403, 0, 7.277565731, 5.381832436, -1.78660692, 0.687665324, 0.5177814627, -0.2500601178, -0.2905436016, 0.1621539124, 0.01082792073, 0.06542071637, -0.001936957344, -0.008217814036, -0.01344291191, 0.005591514219, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8144110179, 0.3863090652, 1.932975571, -0.5257000516, 4.57721453, 1.679445061, 1.939256117, -0.1523623501, -0.7623758464, 0.2073388965, 0, -4.731300464, 3.664849581, 0.2821202523, -1.038014687, -0.3808623405, -0.4397819497, 0.03455253321, 0.172890591, -0.04702004204, 0, 0, 8.187261447, 4.529875319, -1.743549823, 0, 0.4755135882, -0.3683312416, -0.0283541522, 0.1043244013, 0.03827810547, 0.04419974899, -0.003472660249, -0.01737615819, 0.004725692033, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6308400619, -3.391315439, 1.842801771, -0.407205509, 0, 5.010845381, 3.004282657, 1.337552847, -0.7268108201, 0.1606040186, 0, 0, -3.785040371, 3.496470524, 0, -1.13635292, -0.6813072668, -0.3033284742, 0.1648252012, -0.03642156795, 0, 0, 0, 9.356870225, 2.701095772, 0, 0, 0.3804108705, -0.3514085096, 0, 0.1142077656, 0.0684739567, 0.0304856587, -0.01656555601, 0.003660505309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.549509757, 0, -2.327373341, -1.645701472, 0, 0, 6.745368782, 0, 0.9179284245, 0.6490734136, 0, 0, 0, -1.766352173, 0, 0, -1.529705854, 0, -0.2081665999, -0.1471960144, 0, 0, 0, 0, 10.9163486, 0, 0, 0, 0.1775250729, 0, 0, 0.153741223, 0, 0.02092153048, 0.01479375608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9974457174, 1.577100155, -0.5463235193, 0.7726181305, 1.475242111, -0.3809058751, 0.1319496657, 2.799075102, -0.9696280583, -1.142717625, 1.06653645, -0.1287696884, 0.03324819058, -0.01151751107, 4.572230677, -1.180544885, 0.4089527441, -1.669211493, 0.5782318227, 0.5125292426, 0, 0, 0, 0, 0, 5.788311106, -0.6988593941, 0.1804447196, -0.06250788447, -1.075964425, 0.2778128199, -0.09623718381, 0.2977185981, -0.1031327476, -0.08334384597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8796644382, 0.8345229604, 0.4818120558, 0.6813851439, 0.4336804166, 1.142153181, -0.5042642238, 1.481131324, 0.8551315688, -1.007782219, 1.209338662, 1.022077716, -0.2638993315, 0.09141741003, 1.344109479, 3.539885266, -1.562870485, -0.883263699, -0.5099525344, 0.4520083052, 0, 0, 0, 0, 0, 6.56332787, 5.547024474, -1.432235561, 0.4961409519, -0.3163038097, -0.8330267834, 0.3677839464, 0.1575378742, 0.0909545341, -0.07350236324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7434521427, 0.3526503153, 1.018013772, 0.5758755534, -1.466107397, 0.9085135252, 0.3278315802, 0.6258922203, 1.806795209, -0.8517314298, 0, 1.382101328, 0.8921425712, -0.3090472522, -4.543919379, 2.815763853, 1.016051262, -0.3732470366, -1.077471385, 0.3820167423, 0, 0, 0, 0, 0, 0, 7.500946137, 4.841839912, -1.677262546, 1.069302042, -0.6626222403, -0.2391032055, 0.06657190234, 0.1921765287, -0.06212083503, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5758755534, 1.912132318, 1.103970108, 0.4460712856, 0, -1.231530213, 1.06653645, 3.393698204, 1.959352572, -0.6597483286, 0, 0, 1.61245155, 0.5585696018, 0, -3.816892279, 3.305525677, -2.02381138, -1.168448045, 0.2959088962, 0, 0, 0, 0, 0, 0, 0, 8.751103826, 3.03147129, 0, 0.8982137155, -0.7778758957, 0.3609646167, 0.2084030186, -0.0481185919, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.329927623, 0, 2.549509757, -1.030157507, 0, 0, -0.6157651067, 0, 4.524930939, 1.523623501, 0, 0, 0, 1.93494186, 0, 0, -1.908446139, 0, -2.698415173, -0.6833723234, 0, 0, 0, 0, 0, 0, 0, 0, 10.50132459, 0, 0, 0.4491068578, 0, 0.4812861557, 0.111125128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7867957925, 0.7464200273, -0.2154729018, 1.523623501, 0.3878955568, -0.06009252126, 0.01734721666, 2.64952826, -0.764852927, 0.4506939094, 0, 0, 0, 0, 2.257805391, -0.3497777072, 0.1009721267, 3.182314222, -0.9186549864, -1.374126707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.446031515, -0.8436955744, 0.2435539335, -1.549967742, 0.4474371464, 0.451979772, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6649638116, 0.3154200309, 0.3642156795, 1.287696884, 0.4589642123, 0.3555121501, -0.1026275178, 1.119630041, 1.292837411, 0.3809058751, 0, 0, 0, 0, 2.671471366, 2.069312822, -0.5973591574, 1.344773202, 1.552810341, -1.161349033, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.443831388, 4.991370331, -1.440884502, -0.6549809158, -0.756306816, 0.3819926273, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5150787536, -0.7329699161, 0.5642405045, 0.9974457174, 0, 0.5507570547, 0.2384848004, -2.601785101, 2.002855105, 0.2950484222, 0, 0, 0, 0, 0, 3.205765639, 1.388137241, -3.12497053, 2.405603436, -0.8995770926, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.732597666, 3.348313008, 1.522038107, -1.171665481, 0.2958902168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.486904285, 0, -0.407205509, 2.879377767, 0, 0, 0.6884463184, 0, -1.445436167, 0.8517314298, 0, 0, 0, 0, 0, 0, 4.007207049, 0, -1.736094739, -2.596855383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.665747083, 0, 0.8455767263, 0.8541614817, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5758755534, 0.2731617597, -0.05257000516, 1.635594714, 0, 0, 0, 1.346705637, -0.2591736206, 2.125855726, 0, 0, 0, 0, 0, 0, 0, 3.359445449, -0.6465255781, 0.914325241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.7269176, -0.9096957163, -1.470290308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.4460712856, 0.3526503153, 0.2036027545, 1.266926218, 0, 0, 0, 1.738589501, 1.003775116, 1.646680764, 0, 0, 0, 0, 0, 0, 0, 4.337025425, 2.503982797, 0.7082332863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.102424382, 3.523236359, -1.138881976, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-0.5150787536, 0, 0.4702004204, -1.462920386, 0, 0, 0, 0, 2.318119334, -1.901423165, 0, 0, 0, 0, 0, 0, 0, 0, 5.782700567, -0.8177973569, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.136565842, 1.31506763, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3642156795, 0, 0, 1.269541135, 0, 0, 0, 0, 0, 2.503568881, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.46962034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3.138389682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6308400619, 0, 0, 2.198909748, 0, 0, 0, 0, 0, 4.336308502, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.009558712, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5.435850383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.58113883, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9128709292, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.581988897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.479019946, 2.80624304, -0.5400617249, -0.3818813079, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.145643924, 0.7245688373, 2.091650066, -0.2958039892, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6614378278, 0, 1.932183566, -0.1707825128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9354143467, 3.54964787, 0, 2.173706512, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5400617249, 0, 3.54964787, 1.25499004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.90940654, 0, 0, 4.437059837, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.374772708, 2.898275349, -0.8366600265, -0.5916079783, 3.872983346, -0.6, 0.1732050808, -0.4898979486, 0.1414213562, 0.1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.161895004, 1.224744871, 1.414213562, -0.5, 0.6546536707, 3.54964787, -1.024695077, -0.2070196678, -0.2390457219, 0.08451542547, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9, 0.316227766, 2.19089023, -0.3872983346, 0, 1.57116881, 2.38117618, -0.05345224838, -0.3703280399, 0.06546536707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5196152423, 0, 1.58113883, -0.2236067977, 0, 0, 2.945941518, 0, -0.2672612419, 0.0377964473, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.00623059, 1.060660172, -0.2041241452, 1.58771324, 4.535573676, 0, 0, 3.585685828, -0.6900655593, -0.7807200584, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7794228634, 0.2738612788, 0.790569415, 1.229837388, 0, 4.535573676, 0, 0.9258200998, 2.672612419, -0.6047431568, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.45, 0, 0.7302967433, 0.7100469468, 0, 0, 4.535573676, 0, 2.468853599, -0.3491486244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6363961031, -3.130495168, 0, 1.917028951, 0, 0, 0, 5.291502622, 0, 2.592296279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3674234614, 0, -3.130495168, 1.106797181, 0, 0, 0, 0, 5.291502622, 1.496662955, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.857883832, 0, 0, -2.347871376, 0, 0, 0, 0, 0, 6.349803147, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.284523258, 2.901442287, -1.005089091, -0.7107053122, 4.071076113, -1.051147332, 0.3641281172, -0.8582582029, 0.2973093627, 0.2102294665, 4.905354218, -0.5922544268, 0.1529194355, -0.05297284634, -0.5129073792, 0.1324321158, -0.04587583064, 0.1081303698, -0.03745745886, -0.02648642317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.132843031, 1.535298947, 0.8864052604, -0.6267831705, 1.196783885, 3.151884357, -1.391566863, -0.4541475531, -0.262202212, 0.1854049622, 0.6180165406, 4.700873779, -1.213760391, 0.4204589329, -0.1507805968, -0.3971001033, 0.1753209453, 0.05721721353, 0.03303437363, -0.02335882961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9574271078, 0.6487822187, 1.872872943, -0.5297284634, 0.2022933984, 2.507132682, 0.9046835806, -0.1919123684, -0.5540032878, 0.1566957926, 0, 1.412609236, 4.103259033, -1.421410624, -0.02548657258, -0.3158690277, -0.1139794176, 0.02417868573, 0.06979785357, -0.01974181423, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7416198487, 0.1675148486, 2.031009601, -0.4103259033, 0, 0.8496322734, 2.943212531, -0.04955156045, -0.600780742, 0.1213760391, 0, 0, 2.472066162, 2.569046516, 0, -0.1070436048, -0.3708099244, 0.006242909811, 0.07569125885, -0.01529194355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.4281744193, 0, 1.340118789, -0.2369017707, 0, 0, 2.54889682, 0, -0.3964124836, 0.07007648882, 0, 0, 0, 3.95530586, 0, 0, -0.3211308145, 0, 0.04994327848, -0.008828807723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.01324561, 1.373213125, -0.3964124836, 1.121223821, 1.070436048, -0.1658312395, 0.04787135539, 3.249615362, -0.938083152, -1.077903057, 5.527707984, 0, 0, 0, 4.720179297, -0.7312470323, 0.2110928355, -1.023532631, 0.2954684201, 0.2611596544, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8563488386, 0.5802884575, 0.6700593943, 0.947607083, 0.1809367161, 0.9810708435, -0.2832107578, 1.373213125, 1.585649934, -0.9109943547, 0, 5.527707984, 0, 0, 0.7978559231, 4.326115784, -1.248842056, -0.4325214792, -0.4994327848, 0.2207201931, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6633249581, 0.1498298355, 1.03805155, 0.7340132902, 0, 0.4342481187, 0.6581223291, 0.3545621042, 2.456478315, -0.7056531928, 0, 0, 5.527707984, 0, 0, 1.914854216, 2.902046692, -0.1116765657, -0.7737179433, 0.1709691264, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3829708431, 0, 0.7491491773, 0.4237827707, 0, 0, 0.8142152225, 0, 1.772810521, -0.4074090608, 0, 0, 0, 5.527707984, 0, 0, 3.590351654, 0, -0.5583828286, 0.09870907114, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7416198487, 0.5025445457, -0.09671474291, 1.778078914, -4.230786401, 0, 0, 2.22982022, -0.4291291015, 1.33513643, 0, 0, 0, 0, 6.218671482, 0, 0, 4.213964122, -0.8109777734, -1.146895766, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5744562647, 0.1297564437, 0.3745745886, 1.377294005, 0, -4.230786401, 0, 0.5757371052, 1.662009863, 1.034192231, 0, 0, 0, 0, 0, 6.218671482, 0, 1.088040858, 3.140903411, -0.8883816403, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.331662479, 0, 0.3460171833, 0.7951810644, 0, 0, -4.230786401, 0, 1.535298947, 0.5970911632, 0, 0, 0, 0, 0, 0, 6.218671482, 0, 2.901442287, -0.5129073792, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.469041576, 3.814044936, 0, 1.557077325, 0, 0, 0, -3.760699023, 0, 2.763538105, 0, 0, 0, 0, 0, 0, 0, 7.107053122, 0, 2.901442287, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.2708012802, 0, 3.814044936, 0.8989790127, 0, 0, 0, 0, -3.760699023, 1.595529469, 0, 0, 0, 0, 0, 0, 0, 0, 7.107053122, 1.675148486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.680795902, 0, 0, 4.449719092, 0, 0, 0, 0, 0, -2.632489316, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.291561976, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.207966752, 2.864944705, -1.102717731, -0.7797391854, 4.168749479, -1.383899769, 0.5326632696, -1.12994943, 0.4349177384, 0.3075332821, 5.166559139, -1.09163486, 0.3623900255, -0.1394839859, -0.9453835201, 0.3138389682, -0.1207966752, 0.2562484445, -0.09863007227, -0.06974199293, 5.924525297, -0.5805489101, 0.1226633454, -0.0407205509, 0.01567334735, -0.5192587309, 0.1097134314, -0.03642156795, 0.01401866804, 0.09501461876, -0.03154200309, 0.01214052265, -0.02575393768, 0.009912695236, 0.007009334021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.092647039, 1.72762666, 0.4987228587, -0.7053006306, 1.616046764, 2.642656041, -1.445436167, -0.6813851439, -0.1966989481, 0.2781743201, 1.168332145, 4.513924036, -1.96676467, 0.8200920804, -0.3664849581, -0.5992980588, 0.3277941116, 0.1545236261, 0.04460712856, -0.06308400619, 0.5954371961, 5.776388722, -1.220484876, 0.4051643657, -0.1559478371, -0.1174217986, -0.4536664352, 0.1976672867, -0.08242235528, 0.03683312416, 0.06023172117, -0.03294454778, -0.01553020876, -0.004483185103, 0.006340181176, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9636241117, 0.9141741003, 1.495429545, -0.6220166894, 0.4750730938, 2.870322282, -0.08498365856, -0.3605551275, -0.5898053665, 0.2453266907, 0.1471960144, 2.488066758, 2.890872335, -1.613411057, -0.1077364509, -0.6509279091, 0.01927248223, 0.08176621726, 0.1337552847, -0.05563486403, 0, 1.323193769, 5.381832436, -1.78660692, 0.687665324, -0.01479375608, -0.2500601178, -0.2905436016, 0.1621539124, 0.01082792073, 0.06542071637, -0.001936957344, -0.008217814036, -0.01344291191, 0.005591514219, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8144110179, 0.3863090652, 1.932975571, -0.5257000516, 0.0803020093, 1.679445061, 1.939256117, -0.1523623501, -0.7623758464, 0.2073388965, 0, 0.5677560557, 3.664849581, 0.2821202523, -0.01821078398, -0.3808623405, -0.4397819497, 0.03455253321, 0.172890591, -0.04702004204, 0, 0, 2.232889486, 4.529875319, -1.743549823, 0, -0.05706163058, -0.3683312416, -0.0283541522, 0.001830252654, 0.03827810547, 0.04419974899, -0.003472660249, -0.01737615819, 0.004725692033, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6308400619, 0.09974457174, 1.842801771, -0.407205509, 0, 0.5139328596, 3.004282657, -0.03933978962, -0.7268108201, 0.1606040186, 0, 0, 1.514016149, 3.496470524, 0, -0.1165490175, -0.6813072668, 0.008921425712, 0.1648252012, -0.03642156795, 0, 0, 0, 3.402498264, 2.701095772, 0, 0, -0.1521643482, -0.3514085096, 0, 0.01171361699, 0.0684739567, -0.0008966370207, -0.01656555601, 0.003660505309, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3642156795, 0, 1.16368667, -0.2351002102, 0, 0, 2.248456261, 0, -0.4589642123, 0.09272477338, 0, 0, 0, 3.532704347, 0, 0, -0.5099019514, 0, 0.1040833, -0.02102800206, 0, 0, 0, 0, 4.961976634, 0, 0, 0, -0.3550501458, 0, 0, 0.05124707432, 0, -0.01046076524, 0.002113393725, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.9974457174, 1.577100155, -0.5463235193, 0.7726181305, 1.475242111, -0.3809058751, 0.1319496657, 2.799075102, -0.9696280583, -1.142717625, 1.06653645, -0.1287696884, 0.03324819058, -0.01151751107, 4.572230677, -1.180544885, 0.4089527441, -1.669211493, 0.5782318227, 0.5125292426, 6.522687678, 0, 0, 0, 0, 5.788311106, -0.6988593941, 0.1804447196, -0.06250788447, -1.075964425, 0.2778128199, -0.09623718381, 0.2977185981, -0.1031327476, -0.08334384597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.8796644382, 0.8345229604, 0.4818120558, 0.6813851439, 0.4336804166, 1.142153181, -0.5042642238, 1.481131324, 0.8551315688, -1.007782219, 0.1343709625, 1.022077716, -0.2638993315, 0.09141741003, 1.344109479, 3.539885266, -1.562870485, -0.883263699, -0.5099525344, 0.4520083052, 0, 6.522687678, 0, 0, 0, 0.7292586522, 5.547024474, -1.432235561, 0.4961409519, -0.3163038097, -0.8330267834, 0.3677839464, 0.1575378742, 0.0909545341, -0.07350236324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7434521427, 0.3526503153, 1.018013772, 0.5758755534, 0.07330536985, 0.9085135252, 0.3278315802, 0.6258922203, 1.806795209, -0.8517314298, 0, 0.3071336285, 0.8921425712, -0.3090472522, 0.227195969, 2.815763853, 1.016051262, -0.3732470366, -1.077471385, 0.3820167423, 0, 0, 6.522687678, 0, 0, 0, 1.666876919, 4.841839912, -1.677262546, -0.05346510212, -0.6626222403, -0.2391032055, 0.06657190234, 0.1921765287, -0.06212083503, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5758755534, 0.09105391989, 1.103970108, 0.4460712856, 0, 0.3078825534, 1.06653645, 0.1616046764, 1.959352572, -0.6597483286, 0, 0, 0.5374838499, 0.5585696018, 0, 0.9542230697, 3.305525677, -0.09637197045, -1.168448045, 0.2959088962, 0, 0, 0, 6.522687678, 0, 0, 0, 2.917034609, 3.03147129, 0, -0.2245534289, -0.7778758957, 0.01718879127, 0.2084030186, -0.0481185919, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3324819058, 0, 0.7284313591, 0.2575393768, 0, 0, 0.9236476601, 0, 1.292837411, -0.3809058751, 0, 0, 0, 0.8599741598, 0, 0, 2.862669209, 0, -0.7709757636, 0.1708430809, 0, 0, 0, 0, 6.522687678, 0, 0, 0, 4.667255374, 0, 0, -0.6736602867, 0, 0.1375103302, -0.02778128199, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.7867957925, 0.7464200273, -0.2154729018, 1.523623501, 0.3878955568, -0.06009252126, 0.01734721666, 2.64952826, -0.764852927, 0.4506939094, -5.288141871, 0, 0, 0, 2.257805391, -0.3497777072, 0.1009721267, 3.182314222, -0.9186549864, -1.374126707, 0, 0, 0, 0, 0, 7.174956446, 0, 0, 0, 5.446031515, -0.8436955744, 0.2435539335, -1.549967742, 0.4474371464, 0.451979772, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.6649638116, 0.3154200309, 0.3642156795, 1.287696884, 0.06556631604, 0.3555121501, -0.1026275178, 1.119630041, 1.292837411, 0.3809058751, 0, -5.288141871, 0, 0, 0.3816387666, 2.069312822, -0.5973591574, 1.344773202, 1.552810341, -1.161349033, 0, 0, 0, 0, 0, 0, 7.174956446, 0, 0, 0.9205473412, 4.991370331, -1.440884502, -0.6549809158, -0.756306816, 0.3819926273, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5150787536, 0.08144110179, 0.5642405045, 0.9974457174, 0, 0.1573591585, 0.2384848004, 0.2890872335, 2.002855105, 0.2950484222, 0, 0, -5.288141871, 0, 0, 0.9159330398, 1.388137241, 0.3472189478, 2.405603436, -0.8995770926, 0, 0, 0, 0, 0, 0, 0, 7.174956446, 0, 0, 2.209313619, 3.348313008, -0.1691153453, -1.171665481, 0.2958902168, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.2973808571, 0, 0.407205509, 0.5758755534, 0, 0, 0.2950484222, 0, 1.445436167, 0.170346286, 0, 0, 0, -5.288141871, 0, 0, 1.71737445, 0, 1.736094739, -0.5193710766, 0, 0, 0, 0, 0, 0, 0, 0, 7.174956446, 0, 0, 4.142463035, 0, -0.8455767263, 0.1708322963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.5758755534, 0.2731617597, -0.05257000516, 1.635594714, 4.826478102, 0, 0, 1.346705637, -0.2591736206, 2.125855726, 0, 0, 0, 0, -4.957633004, 0, 0, 3.359445449, -0.6465255781, 0.914325241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.972173829, 0, 0, 4.7269176, -0.9096957163, -1.470290308, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.4460712856, 0.07053006306, 0.2036027545, 1.266926218, 0, 4.826478102, 0, 0.3477179002, 1.003775116, 1.646680764, 0, 0, 0, 0, 0, -4.957633004, 0, 0.8674050851, 2.503982797, 0.7082332863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.972173829, 0, 1.220484876, 3.523236359, -1.138881976, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.2575393768, 0, 0.1880801682, 0.7314601928, 0, 0, 4.826478102, 0, 0.9272477338, 0.9507115826, 0, 0, 0, 0, 0, 0, -4.957633004, 0, 2.313080227, 0.4088986785, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7.972173829, 0, 3.254626337, -0.6575338151, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.3642156795, -3.973541319, 0, 1.269541135, 0, 0, 0, 5.110388579, 0, 2.503568881, 0, 0, 0, 0, 0, 0, 0, -4.249399718, 0, 3.46962034, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.968695557, 0, 3.138389682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.2102800206, 0, -3.973541319, 0.7329699161, 0, 0, 0, 0, 5.110388579, 1.445436167, 0, 0, 0, 0, 0, 0, 0, 0, -4.249399718, 2.003186237, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8.968695557, 1.811950128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.271189428, 0, 0, -3.455253321, 0, 0, 0, 0, 0, 6.132466295, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.832933145, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.24993778, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[3];
    
    // Nodal coordinates on reference cell
    static double X[56][3] = {{0, 0, 0}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0, 0.8, 0.2}, {0, 0.6, 0.4}, {0, 0.4, 0.6}, {0, 0.2, 0.8}, {0.8, 0, 0.2}, {0.6, 0, 0.4}, {0.4, 0, 0.6}, {0.2, 0, 0.8}, {0.8, 0.2, 0}, {0.6, 0.4, 0}, {0.4, 0.6, 0}, {0.2, 0.8, 0}, {0, 0, 0.2}, {0, 0, 0.4}, {0, 0, 0.6}, {0, 0, 0.8}, {0, 0.2, 0}, {0, 0.4, 0}, {0, 0.6, 0}, {0, 0.8, 0}, {0.2, 0, 0}, {0.4, 0, 0}, {0.6, 0, 0}, {0.8, 0, 0}, {0.6, 0.2, 0.2}, {0.4, 0.4, 0.2}, {0.2, 0.6, 0.2}, {0.4, 0.2, 0.4}, {0.2, 0.4, 0.4}, {0.2, 0.2, 0.6}, {0, 0.2, 0.2}, {0, 0.4, 0.2}, {0, 0.6, 0.2}, {0, 0.2, 0.4}, {0, 0.4, 0.4}, {0, 0.2, 0.6}, {0.2, 0, 0.2}, {0.4, 0, 0.2}, {0.6, 0, 0.2}, {0.2, 0, 0.4}, {0.4, 0, 0.4}, {0.2, 0, 0.6}, {0.2, 0.2, 0}, {0.4, 0.2, 0}, {0.6, 0.2, 0}, {0.2, 0.4, 0}, {0.4, 0.4, 0}, {0.2, 0.6, 0}, {0.2, 0.2, 0.2}, {0.4, 0.2, 0.2}, {0.2, 0.4, 0.2}, {0.2, 0.2, 0.4}};
    
    // Components for each dof
    static unsigned int components[56] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1] - X[i][2];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    const double w3 = X[i][2];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    coordinates[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new P5tet_finite_element_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class P5tet_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  P5tet_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~P5tet_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 5 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 56;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new P5tet_dof_map_0();
  }

};

#endif
