// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.0.

#ifndef __POISSONDG_H
#define __POISSONDG_H

#include <cmath>
#include <stdexcept>
#include <ufc.h>

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[3][3] = \
    {{0, 0, 0},
    {2.449489743, 0, 0},
    {0, 0, 0}};
    
    const static double dmats1[3][3] = \
    {{0, 0, 0},
    {1.224744871, 0, 0},
    {2.121320344, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[3][2] = {{0, 0}, {1, 0}, {0, 1}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGBilinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[3][3] = \
    {{0, 0, 0},
    {2.449489743, 0, 0},
    {0, 0, 0}};
    
    const static double dmats1[3][3] = \
    {{0, 0, 0},
    {1.224744871, 0, 0},
    {2.121320344, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[3][2] = {{0, 0}, {1, 0}, {0, 1}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGBilinearForm_finite_element_1();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_2_0: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_2_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[1][2] = {{0.3333333333, 0.3333333333}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGBilinearForm_finite_element_2_0();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_2_1: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_2_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[1][2] = {{0.3333333333, 0.3333333333}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGBilinearForm_finite_element_2_1();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a triangle, Discontinuous Lagrange finite element of degree 0 on a triangle]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 2;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 2;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    
    if (0 <= i and i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.414213562}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0;
    }
    
    if (1 <= i and i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.414213562}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0;
    }
    
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 2*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i and i <= 0)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.414213562}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives
      delete [] combinations;
    
    }
    
    if (1 <= i and i <= 1)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 1;
    
      // Generate scalings
      const double scalings_y_0 = 1;
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
      // Table(s) of coefficients
      const static double coefficients0[1][1] =   \
      {{1.414213562}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[1][1] =   \
      {{0}};
    
      const static double dmats1[1][1] =   \
      {{0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives
      delete [] combinations;
    
    }
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[2];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[2][2] = {{0.3333333333, 0.3333333333}, {0.3333333333, 0.3333333333}};
    
    // Components for each dof
    static unsigned int components[2] = {0, 1};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
    // Evaluate at vertices and use affine mapping
    vertex_values[3] = dof_values[1];
    vertex_values[4] = dof_values[1];
    vertex_values[5] = dof_values[1];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 2;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGBilinearForm_finite_element_2_0();
      break;
    case 1:
      return new PoissonDGBilinearForm_finite_element_2_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class PoissonDGBilinearForm_finite_element_3: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGBilinearForm_finite_element_3() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_finite_element_3()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 1;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    
    // Table(s) of coefficients
    const static double coefficients0[1][1] = \
    {{1.414213562}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[1][1] = \
    {{0}};
    
    const static double dmats1[1][1] = \
    {{0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[1][2] = {{0.3333333333, 0.3333333333}};
    
    // Components for each dof
    static unsigned int components[1] = {0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[0];
    vertex_values[2] = dof_values[0];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGBilinearForm_finite_element_3();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 3*c.entity_indices[2][0];
    dofs[1] = 3*c.entity_indices[2][0] + 1;
    dofs[2] = 3*c.entity_indices[2][0] + 2;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGBilinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 3*c.entity_indices[2][0];
    dofs[1] = 3*c.entity_indices[2][0] + 1;
    dofs[2] = 3*c.entity_indices[2][0] + 2;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGBilinearForm_dof_map_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_2_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_2_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_2_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[2][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.3333333333*x[0][0] + 0.3333333333*x[1][0] + 0.3333333333*x[2][0];
    coordinates[0][1] = 0.3333333333*x[0][1] + 0.3333333333*x[1][1] + 0.3333333333*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGBilinearForm_dof_map_2_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_2_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_2_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_2_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[2][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.3333333333*x[0][0] + 0.3333333333*x[1][0] + 0.3333333333*x[2][0];
    coordinates[0][1] = 0.3333333333*x[0][1] + 0.3333333333*x[1][1] + 0.3333333333*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGBilinearForm_dof_map_2_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Discontinuous Lagrange finite element of degree 0 on a triangle, Discontinuous Lagrange finite element of degree 0 on a triangle]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 2*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 2;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[2][0];
    unsigned int offset = m.num_entities[2];
    dofs[1] = offset + c.entity_indices[2][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.3333333333*x[0][0] + 0.3333333333*x[1][0] + 0.3333333333*x[2][0];
    coordinates[0][1] = 0.3333333333*x[0][1] + 0.3333333333*x[1][1] + 0.3333333333*x[2][1];
    coordinates[1][0] = 0.3333333333*x[0][0] + 0.3333333333*x[1][0] + 0.3333333333*x[2][0];
    coordinates[1][1] = 0.3333333333*x[0][1] + 0.3333333333*x[1][1] + 0.3333333333*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 2;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGBilinearForm_dof_map_2_0();
      break;
    case 1:
      return new PoissonDGBilinearForm_dof_map_2_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGBilinearForm_dof_map_3: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGBilinearForm_dof_map_3() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_dof_map_3()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 0 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 1;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[2][0];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.3333333333*x[0][0] + 0.3333333333*x[1][0] + 0.3333333333*x[2][0];
    coordinates[0][1] = 0.3333333333*x[0][1] + 0.3333333333*x[1][1] + 0.3333333333*x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGBilinearForm_dof_map_3();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class PoissonDGBilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  PoissonDGBilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    const double Jinv_00 =  J_11 / detJ;
    const double Jinv_01 = -J_01 / detJ;
    const double Jinv_10 = -J_10 / detJ;
    const double Jinv_11 =  J_00 / detJ;
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Set scale factor
    const double det = detJ;
    
    // Compute geometry tensors
    const double G0_0_0 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01);
    const double G0_0_1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11);
    const double G0_1_0 = det*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01);
    const double G0_1_1 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11);
    
    // Compute element tensor
    A[0] = 0.5*G0_0_0 + 0.5*G0_0_1 + 0.5*G0_1_0 + 0.5*G0_1_1;
    A[1] = -0.5*G0_0_0 - 0.5*G0_1_0;
    A[2] = -0.5*G0_0_1 - 0.5*G0_1_1;
    A[3] = -0.5*G0_0_0 - 0.5*G0_0_1;
    A[4] = 0.5*G0_0_0;
    A[5] = 0.5*G0_0_1;
    A[6] = -0.5*G0_1_0 - 0.5*G0_1_1;
    A[7] = 0.5*G0_1_0;
    A[8] = 0.5*G0_1_1;
  }

};

/// This class defines the interface for the tabulation of the
/// exterior facet tensor corresponding to the local contribution to
/// a form from the integral over an exterior facet.

class PoissonDGBilinearForm_exterior_facet_integral_0: public ufc::exterior_facet_integral
{
public:

  /// Constructor
  PoissonDGBilinearForm_exterior_facet_integral_0() : ufc::exterior_facet_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_exterior_facet_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local exterior facet
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int facet) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    const double Jinv_00 =  J_11 / detJ;
    const double Jinv_01 = -J_01 / detJ;
    const double Jinv_10 = -J_10 / detJ;
    const double Jinv_11 =  J_00 / detJ;
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Vertices on edges
    static unsigned int edge_vertices[3][2] = {{1, 2}, {0, 2}, {0, 1}};
    
    // Get vertices
    const unsigned int v0 = edge_vertices[facet][0];
    const unsigned int v1 = edge_vertices[facet][1];
    
    // Compute scale factor (length of edge scaled by length of reference interval)
    const double dx0 = x[v1][0] - x[v0][0];
    const double dx1 = x[v1][1] - x[v0][1];
    const double det = std::sqrt(dx0*dx0 + dx1*dx1);
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_1_0_0 = w[0][0];
    const double c0_1_0_1 = w[0][1];
    const double c1_2_0_0 = (1.0/w[1][0]);
    
    // Compute geometry tensors
    const double G0_0_0_0 = det*c0_0_0_0*Jinv_00;
    const double G0_0_1_1 = det*c0_0_0_1*Jinv_01;
    const double G0_1_0_0 = det*c0_0_0_0*Jinv_10;
    const double G0_1_1_1 = det*c0_0_0_1*Jinv_11;
    const double G1_0_0_0 = det*c0_1_0_0*Jinv_00;
    const double G1_0_1_0 = det*c0_1_0_0*Jinv_10;
    const double G1_1_0_1 = det*c0_1_0_1*Jinv_01;
    const double G1_1_1_1 = det*c0_1_0_1*Jinv_11;
    const double G2_0 = det*c1_2_0_0;
    
    // Compute element tensor for all facets
    switch ( facet )
    {
    case 0:
      A[0] = 0;
      A[1] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1;
      A[2] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1;
      A[3] = 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1;
      A[4] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1 - 0.5*G1_0_0_0 - 0.5*G1_1_0_1 + 2.666666667*G2_0;
      A[5] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1 - 0.5*G1_0_1_0 - 0.5*G1_1_1_1 + 1.333333333*G2_0;
      A[6] = 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1;
      A[7] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1 - 0.5*G1_0_0_0 - 0.5*G1_1_0_1 + 1.333333333*G2_0;
      A[8] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1 - 0.5*G1_0_1_0 - 0.5*G1_1_1_1 + 2.666666667*G2_0;
      break;
    case 1:
      A[0] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1 + 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1 + 2.666666667*G2_0;
      A[1] = -0.5*G1_0_0_0 - 0.5*G1_1_0_1;
      A[2] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1 - 0.5*G1_0_1_0 - 0.5*G1_1_1_1 + 1.333333333*G2_0;
      A[3] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1;
      A[4] = 0;
      A[5] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1;
      A[6] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1 + 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1 + 1.333333333*G2_0;
      A[7] = -0.5*G1_0_0_0 - 0.5*G1_1_0_1;
      A[8] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1 - 0.5*G1_0_1_0 - 0.5*G1_1_1_1 + 2.666666667*G2_0;
      break;
    case 2:
      A[0] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1 + 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1 + 2.666666667*G2_0;
      A[1] = 0.5*G0_0_0_0 + 0.5*G0_0_1_1 + 0.5*G0_1_0_0 + 0.5*G0_1_1_1 - 0.5*G1_0_0_0 - 0.5*G1_1_0_1 + 1.333333333*G2_0;
      A[2] = -0.5*G1_0_1_0 - 0.5*G1_1_1_1;
      A[3] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1 + 0.5*G1_0_0_0 + 0.5*G1_0_1_0 + 0.5*G1_1_0_1 + 0.5*G1_1_1_1 + 1.333333333*G2_0;
      A[4] = -0.5*G0_0_0_0 - 0.5*G0_0_1_1 - 0.5*G1_0_0_0 - 0.5*G1_1_0_1 + 2.666666667*G2_0;
      A[5] = -0.5*G1_0_1_0 - 0.5*G1_1_1_1;
      A[6] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1;
      A[7] = -0.5*G0_1_0_0 - 0.5*G0_1_1_1;
      A[8] = 0;
      break;
    }
  }

};

/// This class defines the interface for the tabulation of the
/// interior facet tensor corresponding to the local contribution to
/// a form from the integral over an interior facet.

class PoissonDGBilinearForm_interior_facet_integral_0: public ufc::interior_facet_integral
{
public:

  /// Constructor
  PoissonDGBilinearForm_interior_facet_integral_0() : ufc::interior_facet_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm_interior_facet_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local interior facet
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c0,
                               const ufc::cell& c1,
                               unsigned int facet0,
                               unsigned int facet1) const
  {
    // Extract vertex coordinates
    const double * const * x0 = c0.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J0_00 = x0[1][0] - x0[0][0];
    const double J0_01 = x0[2][0] - x0[0][0];
    const double J0_10 = x0[1][1] - x0[0][1];
    const double J0_11 = x0[2][1] - x0[0][1];
      
    // Compute determinant of Jacobian
    double detJ0 = J0_00*J0_11 - J0_01*J0_10;
      
    // Compute inverse of Jacobian
    const double Jinv0_00 =  J0_11 / detJ0;
    const double Jinv0_01 = -J0_01 / detJ0;
    const double Jinv0_10 = -J0_10 / detJ0;
    const double Jinv0_11 =  J0_00 / detJ0;
    
    // Take absolute value of determinant
    detJ0 = std::abs(detJ0);
    
    // Extract vertex coordinates
    const double * const * x1 = c1.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J1_00 = x1[1][0] - x1[0][0];
    const double J1_01 = x1[2][0] - x1[0][0];
    const double J1_10 = x1[1][1] - x1[0][1];
    const double J1_11 = x1[2][1] - x1[0][1];
      
    // Compute determinant of Jacobian
    double detJ1 = J1_00*J1_11 - J1_01*J1_10;
      
    // Compute inverse of Jacobian
    const double Jinv1_00 =  J1_11 / detJ1;
    const double Jinv1_01 = -J1_01 / detJ1;
    const double Jinv1_10 = -J1_10 / detJ1;
    const double Jinv1_11 =  J1_00 / detJ1;
    
    // Take absolute value of determinant
    detJ1 = std::abs(detJ1);
    
    // Vertices on edges
    static unsigned int edge_vertices[3][2] = {{1, 2}, {0, 2}, {0, 1}};
    
    // Get vertices
    const unsigned int v0 = edge_vertices[facet0][0];
    const unsigned int v1 = edge_vertices[facet0][1];
    
    // Compute scale factor (length of edge scaled by length of reference interval)
    const double dx0 = x0[v1][0] - x0[v0][0];
    const double dx1 = x0[v1][1] - x0[v0][1];
    const double det = std::sqrt(dx0*dx0 + dx1*dx1);
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_1_0_2 = w[0][2];
    const double c0_1_0_3 = w[0][3];
    const double c0_2_0_0 = w[0][0];
    const double c0_2_0_1 = w[0][1];
    const double c0_3_0_2 = w[0][2];
    const double c0_3_0_3 = w[0][3];
    const double c0_4_0_0 = w[0][0];
    const double c0_4_0_1 = w[0][1];
    const double c0_5_0_0 = w[0][0];
    const double c0_5_0_1 = w[0][1];
    const double c0_6_0_2 = w[0][2];
    const double c0_6_0_3 = w[0][3];
    const double c0_7_0_2 = w[0][2];
    const double c0_7_0_3 = w[0][3];
    const double c1_8_0_0 = (1.0/w[1][0]);
    const double c0_8_1_0 = w[0][0];
    const double c0_8_1_1 = w[0][1];
    const double c0_8_2_0 = w[0][0];
    const double c0_8_2_1 = w[0][1];
    const double c1_9_0_0 = (1.0/w[1][0]);
    const double c0_9_1_0 = w[0][0];
    const double c0_9_1_1 = w[0][1];
    const double c0_9_2_2 = w[0][2];
    const double c0_9_2_3 = w[0][3];
    const double c1_10_0_0 = (1.0/w[1][0]);
    const double c0_10_1_2 = w[0][2];
    const double c0_10_1_3 = w[0][3];
    const double c0_10_2_0 = w[0][0];
    const double c0_10_2_1 = w[0][1];
    const double c1_11_0_0 = (1.0/w[1][0]);
    const double c0_11_1_2 = w[0][2];
    const double c0_11_1_3 = w[0][3];
    const double c0_11_2_2 = w[0][2];
    const double c0_11_2_3 = w[0][3];
    
    // Compute geometry tensors
    const double G0_0_0_0 = det*c0_0_0_0*Jinv0_00;
    const double G0_0_1_1 = det*c0_0_0_1*Jinv0_01;
    const double G0_1_0_0 = det*c0_0_0_0*Jinv0_10;
    const double G0_1_1_1 = det*c0_0_0_1*Jinv0_11;
    const double G1_0_2_0 = det*c0_1_0_2*Jinv0_00;
    const double G1_0_3_1 = det*c0_1_0_3*Jinv0_01;
    const double G1_1_2_0 = det*c0_1_0_2*Jinv0_10;
    const double G1_1_3_1 = det*c0_1_0_3*Jinv0_11;
    const double G2_0_0_0 = det*c0_2_0_0*Jinv1_00;
    const double G2_0_1_1 = det*c0_2_0_1*Jinv1_01;
    const double G2_1_0_0 = det*c0_2_0_0*Jinv1_10;
    const double G2_1_1_1 = det*c0_2_0_1*Jinv1_11;
    const double G3_0_2_0 = det*c0_3_0_2*Jinv1_00;
    const double G3_0_3_1 = det*c0_3_0_3*Jinv1_01;
    const double G3_1_2_0 = det*c0_3_0_2*Jinv1_10;
    const double G3_1_3_1 = det*c0_3_0_3*Jinv1_11;
    const double G4_0_0_0 = det*c0_4_0_0*Jinv0_00;
    const double G4_0_1_0 = det*c0_4_0_0*Jinv0_10;
    const double G4_1_0_1 = det*c0_4_0_1*Jinv0_01;
    const double G4_1_1_1 = det*c0_4_0_1*Jinv0_11;
    const double G5_0_0_0 = det*c0_5_0_0*Jinv1_00;
    const double G5_0_1_0 = det*c0_5_0_0*Jinv1_10;
    const double G5_1_0_1 = det*c0_5_0_1*Jinv1_01;
    const double G5_1_1_1 = det*c0_5_0_1*Jinv1_11;
    const double G6_2_0_0 = det*c0_6_0_2*Jinv0_00;
    const double G6_2_1_0 = det*c0_6_0_2*Jinv0_10;
    const double G6_3_0_1 = det*c0_6_0_3*Jinv0_01;
    const double G6_3_1_1 = det*c0_6_0_3*Jinv0_11;
    const double G7_2_0_0 = det*c0_7_0_2*Jinv1_00;
    const double G7_2_1_0 = det*c0_7_0_2*Jinv1_10;
    const double G7_3_0_1 = det*c0_7_0_3*Jinv1_01;
    const double G7_3_1_1 = det*c0_7_0_3*Jinv1_11;
    const double G8_0_0_0 = det*c1_8_0_0*c0_8_1_0*c0_8_2_0;
    const double G8_0_1_1 = det*c1_8_0_0*c0_8_1_1*c0_8_2_1;
    const double G9_0_0_2 = det*c1_9_0_0*c0_9_1_0*c0_9_2_2;
    const double G9_0_1_3 = det*c1_9_0_0*c0_9_1_1*c0_9_2_3;
    const double G10_0_2_0 = det*c1_10_0_0*c0_10_1_2*c0_10_2_0;
    const double G10_0_3_1 = det*c1_10_0_0*c0_10_1_3*c0_10_2_1;
    const double G11_0_2_2 = det*c1_11_0_0*c0_11_1_2*c0_11_2_2;
    const double G11_0_3_3 = det*c1_11_0_0*c0_11_1_3*c0_11_2_3;
    
    // Compute element tensor for all facet-facet combinations
    switch ( facet0 )
    {
    case 0:
      switch ( facet1 )
      {
      case 0:
        A[0] = 0;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[3] = 0;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[6] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[9] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[12] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[18] = 0;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[21] = 0;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[24] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[27] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[30] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[33] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 1:
        A[0] = 0;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[4] = 0;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[6] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[10] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[12] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[16] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[18] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[24] = 0;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[28] = 0;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[30] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[34] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 2:
        A[0] = 0;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1;
        A[5] = 0;
        A[6] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[11] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[12] = 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[17] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[18] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[23] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[24] = 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[30] = 0;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[35] = 0;
        break;
      }
      break;
    case 1:
      switch ( facet1 )
      {
      case 0:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[3] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[7] = 0;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[9] = 0;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[13] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[19] = 0;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[21] = 0;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[25] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[27] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[31] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[33] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 1:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[4] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[7] = 0;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[10] = 0;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[13] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[16] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[19] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[25] = 0;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[28] = 0;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[31] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[34] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 2:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[2] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[5] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[7] = 0;
        A[8] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1;
        A[11] = 0;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[13] = -0.25*G4_0_0_0 - 0.25*G4_1_0_1;
        A[14] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1 - 0.25*G4_0_1_0 - 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[17] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[19] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[20] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[23] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[25] = -0.25*G6_2_0_0 - 0.25*G6_3_0_1;
        A[26] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_1_0 - 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[31] = 0;
        A[32] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[35] = 0;
        break;
      }
      break;
    case 2:
      switch ( facet1 )
      {
      case 0:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[2] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[3] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[9] = 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[14] = 0;
        A[15] = 0;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1;
        A[20] = 0;
        A[21] = 0;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[26] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[27] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[32] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[33] = 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 1:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[2] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[4] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[5] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[10] = -0.25*G5_0_0_0 - 0.25*G5_1_0_1;
        A[11] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_1_0 - 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[14] = 0;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[16] = 0;
        A[17] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[20] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[23] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1;
        A[26] = 0;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[28] = 0;
        A[29] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[32] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[34] = -0.25*G7_2_0_0 - 0.25*G7_3_0_1;
        A[35] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1 - 0.25*G7_2_1_0 - 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        break;
      case 2:
        A[0] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[1] = 0.25*G0_0_0_0 + 0.25*G0_0_1_1 + 0.25*G0_1_0_0 + 0.25*G0_1_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[2] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[3] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[4] = 0.25*G1_0_2_0 + 0.25*G1_0_3_1 + 0.25*G1_1_2_0 + 0.25*G1_1_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[5] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[6] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 + 0.25*G4_0_0_0 + 0.25*G4_0_1_0 + 0.25*G4_1_0_1 + 0.25*G4_1_1_1 + 0.6666666667*G8_0_0_0 + 0.6666666667*G8_0_1_1;
        A[7] = -0.25*G0_0_0_0 - 0.25*G0_0_1_1 - 0.25*G4_0_0_0 - 0.25*G4_1_0_1 + 1.333333333*G8_0_0_0 + 1.333333333*G8_0_1_1;
        A[8] = -0.25*G4_0_1_0 - 0.25*G4_1_1_1;
        A[9] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 + 0.25*G5_0_0_0 + 0.25*G5_0_1_0 + 0.25*G5_1_0_1 + 0.25*G5_1_1_1 + 0.6666666667*G9_0_0_2 + 0.6666666667*G9_0_1_3;
        A[10] = -0.25*G1_0_2_0 - 0.25*G1_0_3_1 - 0.25*G5_0_0_0 - 0.25*G5_1_0_1 + 1.333333333*G9_0_0_2 + 1.333333333*G9_0_1_3;
        A[11] = -0.25*G5_0_1_0 - 0.25*G5_1_1_1;
        A[12] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[13] = -0.25*G0_1_0_0 - 0.25*G0_1_1_1;
        A[14] = 0;
        A[15] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[16] = -0.25*G1_1_2_0 - 0.25*G1_1_3_1;
        A[17] = 0;
        A[18] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[19] = 0.25*G2_0_0_0 + 0.25*G2_0_1_1 + 0.25*G2_1_0_0 + 0.25*G2_1_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[20] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[21] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[22] = 0.25*G3_0_2_0 + 0.25*G3_0_3_1 + 0.25*G3_1_2_0 + 0.25*G3_1_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[23] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[24] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 + 0.25*G6_2_0_0 + 0.25*G6_2_1_0 + 0.25*G6_3_0_1 + 0.25*G6_3_1_1 + 0.6666666667*G10_0_2_0 + 0.6666666667*G10_0_3_1;
        A[25] = -0.25*G2_0_0_0 - 0.25*G2_0_1_1 - 0.25*G6_2_0_0 - 0.25*G6_3_0_1 + 1.333333333*G10_0_2_0 + 1.333333333*G10_0_3_1;
        A[26] = -0.25*G6_2_1_0 - 0.25*G6_3_1_1;
        A[27] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 + 0.25*G7_2_0_0 + 0.25*G7_2_1_0 + 0.25*G7_3_0_1 + 0.25*G7_3_1_1 + 0.6666666667*G11_0_2_2 + 0.6666666667*G11_0_3_3;
        A[28] = -0.25*G3_0_2_0 - 0.25*G3_0_3_1 - 0.25*G7_2_0_0 - 0.25*G7_3_0_1 + 1.333333333*G11_0_2_2 + 1.333333333*G11_0_3_3;
        A[29] = -0.25*G7_2_1_0 - 0.25*G7_3_1_1;
        A[30] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[31] = -0.25*G2_1_0_0 - 0.25*G2_1_1_1;
        A[32] = 0;
        A[33] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[34] = -0.25*G3_1_2_0 - 0.25*G3_1_3_1;
        A[35] = 0;
        break;
      }
      break;
    }
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class PoissonDGBilinearForm: public ufc::form
{
public:

  /// Constructor
  PoissonDGBilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGBilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "(dXa0/dxb0)(dXa1/dxb0) | ((d/dXa0)vi0)*((d/dXa1)vi1)*dX(0) + -0.5w0_a1(dXa0/dxa2)(+) | ((d/dXa0)vi0(+))*vi1(+)*va1[a2](+)*dS(0) + -0.5w0_a1(dXa0/dxa2)(+) | ((d/dXa0)vi0(+))*vi1(-)*va1[a2](-)*dS(0) + -0.5w0_a1(dXa0/dxa2)(-) | ((d/dXa0)vi0(-))*vi1(+)*va1[a2](+)*dS(0) + -0.5w0_a1(dXa0/dxa2)(-) | ((d/dXa0)vi0(-))*vi1(-)*va1[a2](-)*dS(0) + -0.5w0_a0(dXa1/dxa2)(+) | vi0(+)*va0[a2](+)*((d/dXa1)vi1(+))*dS(0) + -0.5w0_a0(dXa1/dxa2)(-) | vi0(+)*va0[a2](+)*((d/dXa1)vi1(-))*dS(0) + -0.5w0_a0(dXa1/dxa2)(+) | vi0(-)*va0[a2](-)*((d/dXa1)vi1(+))*dS(0) + -0.5w0_a0(dXa1/dxa2)(-) | vi0(-)*va0[a2](-)*((d/dXa1)vi1(-))*dS(0) + 4.0([inv]w1_a0)w0_a1w0_a2 | va0(+)*vi0(+)*va1[b0](+)*vi1(+)*va2[b0](+)*dS(0) + 4.0([inv]w1_a0)w0_a1w0_a2 | va0(+)*vi0(+)*va1[b0](+)*vi1(-)*va2[b0](-)*dS(0) + 4.0([inv]w1_a0)w0_a1w0_a2 | va0(+)*vi0(-)*va1[b0](-)*vi1(+)*va2[b0](+)*dS(0) + 4.0([inv]w1_a0)w0_a1w0_a2 | va0(+)*vi0(-)*va1[b0](-)*vi1(-)*va2[b0](-)*dS(0) + -w0_a1(dXa0/dxa2) | ((d/dXa0)vi0)*vi1*va1[a2]*ds(0) + -w0_a0(dXa1/dxa2) | vi0*va0[a2]*((d/dXa1)vi1)*ds(0) + 8.0([inv]w1_a0) | va0*vi0*vi1*ds(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 2;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 1;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 1;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGBilinearForm_finite_element_0();
      break;
    case 1:
      return new PoissonDGBilinearForm_finite_element_1();
      break;
    case 2:
      return new PoissonDGBilinearForm_finite_element_2();
      break;
    case 3:
      return new PoissonDGBilinearForm_finite_element_3();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGBilinearForm_dof_map_0();
      break;
    case 1:
      return new PoissonDGBilinearForm_dof_map_1();
      break;
    case 2:
      return new PoissonDGBilinearForm_dof_map_2();
      break;
    case 3:
      return new PoissonDGBilinearForm_dof_map_3();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new PoissonDGBilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return new PoissonDGBilinearForm_exterior_facet_integral_0();
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return new PoissonDGBilinearForm_interior_facet_integral_0();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGLinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGLinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[3][3] = \
    {{0, 0, 0},
    {2.449489743, 0, 0},
    {0, 0, 0}};
    
    const static double dmats1[3][3] = \
    {{0, 0, 0},
    {1.224744871, 0, 0},
    {2.121320344, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[3][2] = {{0, 0}, {1, 0}, {0, 1}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGLinearForm_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGLinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGLinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[3][3] = \
    {{0, 0, 0},
    {2.449489743, 0, 0},
    {0, 0, 0}};
    
    const static double dmats1[3][3] = \
    {{0, 0, 0},
    {1.224744871, 0, 0},
    {2.121320344, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[3][2] = {{0, 0}, {1, 0}, {0, 1}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGLinearForm_finite_element_1();
  }

};

/// This class defines the interface for a finite element.

class PoissonDGLinearForm_finite_element_2: public ufc::finite_element
{
public:

  /// Constructor
  PoissonDGLinearForm_finite_element_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_finite_element_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::triangle;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 3;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2;
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute constants
    const double C0 = element_coordinates[1][0] + element_coordinates[2][0];
    const double C1 = element_coordinates[1][1] + element_coordinates[2][1];
    
    // Get coordinates and map to the reference (FIAT) element
    double x = (J_01*C1 - J_11*C0 + 2.0*J_11*coordinates[0] - 2.0*J_01*coordinates[1]) / detJ;
    double y = (J_10*C0 - J_00*C1 - 2.0*J_10*coordinates[0] + 2.0*J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-09)
      x = -1.0;
    else
      x = 2.0 * (1.0 + x)/(1.0 - y) - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian, components are scaled because of difference in FFC/FIAT reference elements
    const double Jinv[2][2] =  {{2*J_11 / detJ, -2*J_01 / detJ}, {-2*J_10 / detJ, 2*J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.7071067812*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.732050808*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    
    // Table(s) of coefficients
    const static double coefficients0[3][3] = \
    {{0.4714045208, -0.2886751346, -0.1666666667},
    {0.4714045208, 0.2886751346, -0.1666666667},
    {0.4714045208, 0, 0.3333333333}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[3][3] = \
    {{0, 0, 0},
    {2.449489743, 0, 0},
    {0, 0, 0}};
    
    const static double dmats1[3][3] = \
    {{0, 0, 0},
    {1.224744871, 0, 0},
    {2.121320344, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives
    delete [] combinations;
    
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    double values[1];
    double coordinates[2];
    
    // Nodal coordinates on reference cell
    static double X[3][2] = {{0, 0}, {1, 0}, {0, 1}};
    
    // Components for each dof
    static unsigned int components[3] = {0, 0, 0};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0] - X[i][1];
    const double w1 = X[i][0];
    const double w2 = X[i][1];
    
    // Compute affine mapping x = F(X)
    coordinates[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    coordinates[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at coordinates
    f.evaluate(values, coordinates, c);
    
    // Pick component for evaluation
    return values[components[i]];
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new PoissonDGLinearForm_finite_element_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGLinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGLinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 3*c.entity_indices[2][0];
    dofs[1] = 3*c.entity_indices[2][0] + 1;
    dofs[2] = 3*c.entity_indices[2][0] + 2;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGLinearForm_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGLinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGLinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 3*c.entity_indices[2][0];
    dofs[1] = 3*c.entity_indices[2][0] + 1;
    dofs[2] = 3*c.entity_indices[2][0] + 2;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGLinearForm_dof_map_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class PoissonDGLinearForm_dof_map_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  PoissonDGLinearForm_dof_map_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_dof_map_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Discontinuous Lagrange finite element of degree 1 on a triangle";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return false;
      break;
    case 2:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[2];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 0;
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = 3*c.entity_indices[2][0];
    dofs[1] = 3*c.entity_indices[2][0] + 1;
    dofs[2] = 3*c.entity_indices[2][0] + 2;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      
      break;
    case 1:
      
      break;
    case 2:
      
      break;
    }
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new PoissonDGLinearForm_dof_map_2();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class PoissonDGLinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  PoissonDGLinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Set scale factor
    const double det = detJ;
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_0_0_2 = w[0][2];
    
    // Compute geometry tensors
    const double G0_0 = det*c0_0_0_0;
    const double G0_1 = det*c0_0_0_1;
    const double G0_2 = det*c0_0_0_2;
    
    // Compute element tensor
    A[0] = 0.08333333333*G0_0 + 0.04166666667*G0_1 + 0.04166666667*G0_2;
    A[1] = 0.04166666667*G0_0 + 0.08333333333*G0_1 + 0.04166666667*G0_2;
    A[2] = 0.04166666667*G0_0 + 0.04166666667*G0_1 + 0.08333333333*G0_2;
  }

};

/// This class defines the interface for the tabulation of the
/// exterior facet tensor corresponding to the local contribution to
/// a form from the integral over an exterior facet.

class PoissonDGLinearForm_exterior_facet_integral_0: public ufc::exterior_facet_integral
{
public:

  /// Constructor
  PoissonDGLinearForm_exterior_facet_integral_0() : ufc::exterior_facet_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm_exterior_facet_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local exterior facet
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c,
                               unsigned int facet) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    
    // Take absolute value of determinant
    detJ = std::abs(detJ);
    
    // Vertices on edges
    static unsigned int edge_vertices[3][2] = {{1, 2}, {0, 2}, {0, 1}};
    
    // Get vertices
    const unsigned int v0 = edge_vertices[facet][0];
    const unsigned int v1 = edge_vertices[facet][1];
    
    // Compute scale factor (length of edge scaled by length of reference interval)
    const double dx0 = x[v1][0] - x[v0][0];
    const double dx1 = x[v1][1] - x[v0][1];
    const double det = std::sqrt(dx0*dx0 + dx1*dx1);
    
    // Compute coefficients
    const double c1_0_0_0 = w[1][0];
    const double c1_0_0_1 = w[1][1];
    const double c1_0_0_2 = w[1][2];
    
    // Compute geometry tensors
    const double G0_0 = det*c1_0_0_0;
    const double G0_1 = det*c1_0_0_1;
    const double G0_2 = det*c1_0_0_2;
    
    // Compute element tensor for all facets
    switch ( facet )
    {
    case 0:
      A[0] = 0;
      A[1] = 0.3333333333*G0_1 + 0.1666666667*G0_2;
      A[2] = 0.1666666667*G0_1 + 0.3333333333*G0_2;
      break;
    case 1:
      A[0] = 0.3333333333*G0_0 + 0.1666666667*G0_2;
      A[1] = 0;
      A[2] = 0.1666666667*G0_0 + 0.3333333333*G0_2;
      break;
    case 2:
      A[0] = 0.3333333333*G0_0 + 0.1666666667*G0_1;
      A[1] = 0.1666666667*G0_0 + 0.3333333333*G0_1;
      A[2] = 0;
      break;
    }
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class PoissonDGLinearForm: public ufc::form
{
public:

  /// Constructor
  PoissonDGLinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~PoissonDGLinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w0_a0 | vi0*va0*dX(0) + w1_a0 | vi0*va0*ds(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 2;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 1;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGLinearForm_finite_element_0();
      break;
    case 1:
      return new PoissonDGLinearForm_finite_element_1();
      break;
    case 2:
      return new PoissonDGLinearForm_finite_element_2();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new PoissonDGLinearForm_dof_map_0();
      break;
    case 1:
      return new PoissonDGLinearForm_dof_map_1();
      break;
    case 2:
      return new PoissonDGLinearForm_dof_map_2();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new PoissonDGLinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return new PoissonDGLinearForm_exterior_facet_integral_0();
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

#endif
